@@ -4,27 +4,25 @@
 # This is open-source software licensed under a BSD license.
 # Please see the file LICENSE.txt for details.
 #
-import sys, os
-import re
+import sys
 import math
-import logging
-import time
 import traceback
 
-import numpy, numpy.ma
+import numpy
 
-from ginga.util import wcsmod, io_fits, iohelper
+from ginga.util import wcsmod, io_fits
 from ginga.util import wcs, iqcalc
 from ginga.BaseImage import BaseImage, ImageError, Header
 from ginga.misc import Bunch
 from ginga import trcalc
 import ginga.util.six as six
-from ginga.util.six.moves import map, zip
+from ginga.util.six.moves import map
 
 
 class AstroHeader(Header):
     pass
 
+
 class AstroImage(BaseImage):
     """
     Abstraction of an astronomical data (image).
@@ -43,7 +41,6 @@ class AstroImage(BaseImage):
     def set_ioClass(cls, klass):
         cls.ioClass = klass
 
-
     def __init__(self, data_np=None, metadata=None, logger=None,
                  name=None, wcsclass=wcsClass, ioclass=ioClass,
                  inherit_primary_header=False):
@@ -56,10 +53,11 @@ class AstroImage(BaseImage):
             wcsclass = wcsmod.WCS
         self.wcs = wcsclass(self.logger)
 
-        # wcsclass specifies a pluggable IO module
+        # ioclass specifies a pluggable IO module
         if ioclass is None:
             ioclass = io_fits.fitsLoaderClass
         self.io = ioclass(self.logger)
+        self.io.register_type('image', self.__class__)
 
         self.inherit_primary_header = inherit_primary_header
         if self.inherit_primary_header:
@@ -77,91 +75,89 @@ class AstroImage(BaseImage):
         self.revnaxis = []
         self._md_data = None
 
-    def load_hdu(self, hdu, fobj=None, naxispath=None):
-        self.clear_metadata()
+    def setup_data(self, data, naxispath=None):
+        # initialize data attribute to something reasonable
+        if data is None:
+            data = numpy.zeros((0, 0))
+        elif not isinstance(data, numpy.ndarray):
+            data = numpy.zeros((0, 0))
+        elif 0 in data.shape:
+            data = numpy.zeros((0, 0))
+        elif len(data.shape) < 2:
+            # Expand 1D arrays into 1xN array
+            data = data.reshape((1, data.shape[0]))
 
-        ahdr = self.get_header()
-
-        loader = io_fits.PyFitsFileHandler(self.logger)
-        _data, naxispath = loader.load_hdu(hdu, ahdr, naxispath=naxispath)
         # this is a handle to the full data array
-        self._md_data = _data
+        self._md_data = data
+
+        # this will get reset in set_naxispath() if array is
+        # multidimensional
+        self._data = data
 
         if naxispath is None:
             naxispath = []
 
-        # Drill down to 2D data slice
+        # Set naxispath to drill down to first 2D data slice
         if len(naxispath) == 0:
-            naxispath = ([0] * (len(_data.shape)-2))
+            naxispath = ([0] * (len(data.shape) - 2))
 
         self.set_naxispath(naxispath)
 
+
+    def load_hdu(self, hdu, fobj=None, naxispath=None,
+                 inherit_primary_header=None):
+
+        if self.io is None:
+            # need image loader for the fromHDU() call below
+            raise ImageError("No IO loader defined")
+
+        self.clear_metadata()
+
+        # collect HDU header
+        ahdr = self.get_header()
+        self.io.fromHDU(hdu, ahdr)
+
         # Set PRIMARY header
-        if self.inherit_primary_header and fobj is not None:
+        if inherit_primary_header is None:
+            inherit_primary_header = self.inherit_primary_header
+        else:  # This ensures get_header() is consistent
+            self.inherit_primary_header = inherit_primary_header
+
+        if inherit_primary_header and (fobj is not None):
+            if self._primary_hdr is None:
+                self._primary_hdr = AstroHeader()
+
             self.io.fromHDU(fobj[0], self._primary_hdr)
 
+        self.setup_data(hdu.data)
+
         # Try to make a wcs object on the header
         self.wcs.load_header(hdu.header, fobj=fobj)
 
-    def load_file(self, filepath, numhdu=None, naxispath=None,
-                  allow_numhdu_override=True, memmap=None):
-        self.logger.debug("Loading file '%s' ..." % (filepath))
+    def load_file(self, filespec, **kwargs):
+
+        if self.io is None:
+            raise ImageError("No IO loader defined")
+
+        self.io.load_file(filespec, dstobj=self, **kwargs)
+
+    def load_data(self, data_np, naxispath=None, metadata=None):
+
         self.clear_metadata()
 
-        ahdr = self.get_header()
+        self.setup_data(data_np, naxispath=naxispath)
 
-        info = iohelper.get_fileinfo(filepath)
-        if numhdu is None:
-            numhdu = info.numhdu
+        if metadata is not None:
+            self.update_metadata(metadata)
 
-        _data, numhdu_, naxispath = self.io.load_file(info.filepath, ahdr,
-                                                      numhdu=numhdu,
-                                                      naxispath=naxispath,
-                                                      phdr=self._primary_hdr,
-                                                      memmap=memmap)
-        # this is a handle to the full data array
-        self._md_data = _data
-
-        if naxispath is None:
-            naxispath = []
-
-        # Drill down to 2D data slice
-        if len(naxispath) == 0:
-            naxispath = ([0] * (len(_data.shape)-2))
-
-        # Set the image name if no name currently exists for this image
-        # TODO: should this *change* the existing name, if any?
-        if not (self.name is None):
-            self.set(name=self.name)
-        else:
-            name = self.get('name', None)
-            if name is None:
-                name = info.name
-                if ('[' not in name):
-                    if (numhdu is None) or allow_numhdu_override:
-                        numhdu = numhdu_
-                    name += iohelper.get_hdu_suffix(numhdu)
-                self.set(name=name)
-
-        self.set(path=filepath, idx=numhdu)
-
-        self.set_naxispath(naxispath)
-
-        # Try to make a wcs object on the header
-        # TODO: in order to do more sophisticated WCS (e.g. distortion
-        #   correction) that requires info in additional headers we need
-        #   to pass additional information to the wcs class
-        #self.wcs.load_header(hdu.header, fobj=fobj)
-        self.wcs.load_header(ahdr)
-
-
-    def load_buffer(self, data, dims, dtype, byteswap=False,
-                    metadata=None):
-        data = numpy.fromstring(data, dtype=dtype)
+    def load_buffer(self, buf, dims, dtype, byteswap=False,
+                    naxispath=None, metadata=None):
+        data = numpy.fromstring(buf, dtype=dtype)
         if byteswap:
             data.byteswap(True)
         data = data.reshape(dims)
-        self.set_data(data, metadata=metadata)
+
+        self.load_data(data, naxispath=naxispath, metadata=metadata)
 
     def get_mddata(self):
         return self._md_data
@@ -176,9 +172,9 @@ class AstroImage(BaseImage):
         view = revnaxis + [slice(None), slice(None)]
         data = self.get_mddata()[view]
 
-        assert len(data.shape) == 2, \
-               ImageError("naxispath does not lead to a 2D slice: %s" % (
-            str(naxispath)))
+        if len(data.shape) != 2:
+            raise ImageError(
+                "naxispath does not lead to a 2D slice: {}".format(naxispath))
 
         self.naxispath = naxispath
         self.revnaxis = revnaxis
@@ -219,7 +215,6 @@ class AstroImage(BaseImage):
         except KeyError as e:
             if not create:
                 raise e
-            #hdr = {}
             hdr = AstroHeader()
             self.metadata['header'] = hdr
             displayhdr = hdr
@@ -244,7 +239,7 @@ class AstroImage(BaseImage):
         kwds = self.get_header(create=create)
         kwd = kwd.upper()
         if not create:
-            prev = kwds[kwd]
+            prev = kwds[kwd]  # noqa, this raises KeyError
         kwds[kwd] = value
 
     def update_keywords(self, keyDict):
@@ -267,8 +262,8 @@ class AstroImage(BaseImage):
         self.set_data(data_np.copy(), metadata=metadata,
                       astype=astype)
 
-    def update_metadata(self, keyDict):
-        for key, val in keyDict.items():
+    def update_metadata(self, key_dict):
+        for key, val in key_dict.items():
             self.metadata[key] = val
 
         # refresh the WCS
@@ -276,8 +271,12 @@ class AstroImage(BaseImage):
             header = self.get_header()
             self.wcs.load_header(header)
 
-    def clear_metadata(self):
-        self.metadata = {}
+    def clear_all(self):
+        # clear metadata and data
+        super(AstroImage, self).clear_all()
+
+        # unreference full data array
+        self._md_data = self._data
 
     def transfer(self, other, astype=None):
         data = self._get_data()
@@ -319,7 +318,7 @@ class AstroImage(BaseImage):
         return self.wcs.radectopix(ra_deg, dec_deg, coords=coords,
                                    naxispath=self.revnaxis)
 
-    #-----> TODO: merge into wcs.py ?
+    # -----> TODO: merge into wcs.py ?
     #
     def get_starsep_XY(self, x1, y1, x2, y2):
         # source point
@@ -373,7 +372,6 @@ class AstroImage(BaseImage):
                                    float(self.height / 2.0),
                                    delta_deg)
 
-
     def calc_compass(self, x, y, len_deg_e, len_deg_n):
 
         # Get east and north coordinates
@@ -412,7 +410,7 @@ class AstroImage(BaseImage):
 
         return self.calc_compass_radius(x, y, radius_px)
     #
-    #<----- TODO: merge this into wcs.py ?
+    # <----- TODO: merge this into wcs.py ?
 
     def get_wcs_rotation_deg(self):
         header = self.get_header()
@@ -439,7 +437,6 @@ class AstroImage(BaseImage):
         header = self.get_header()
         ((xrot_ref, yrot_ref),
          (cdelt1_ref, cdelt2_ref)) = wcs.get_xy_rotation_and_scale(header)
-        ref_rot = yrot_ref
 
         scale_x, scale_y = math.fabs(cdelt1_ref), math.fabs(cdelt2_ref)
 
@@ -472,7 +469,6 @@ class AstroImage(BaseImage):
             if bg_ref is not None:
                 bg = iqcalc.get_median(data_np)
                 bg_inc = bg_ref - bg
-                #print "bg=%f inc=%f" % (bg, bg_inc)
                 data_np = data_np + bg_inc
 
             # Determine max/min to update our values
@@ -486,19 +482,20 @@ class AstroImage(BaseImage):
             header = image.get_header()
             ((xrot, yrot),
              (cdelt1, cdelt2)) = wcs.get_xy_rotation_and_scale(header)
-            self.logger.debug("image(%s) xrot=%f yrot=%f cdelt1=%f cdelt2=%f" % (
-                name, xrot, yrot, cdelt1, cdelt2))
+            self.logger.debug("image(%s) xrot=%f yrot=%f cdelt1=%f "
+                              "cdelt2=%f" % (name, xrot, yrot, cdelt1, cdelt2))
 
             # scale if necessary
             # TODO: combine with rotation?
             if (not numpy.isclose(math.fabs(cdelt1), scale_x) or
-                not numpy.isclose(math.fabs(cdelt2), scale_y)):
+                    not numpy.isclose(math.fabs(cdelt2), scale_y)):
                 nscale_x = math.fabs(cdelt1) / scale_x
                 nscale_y = math.fabs(cdelt2) / scale_y
                 self.logger.debug("scaling piece by x(%f), y(%f)" % (
                     nscale_x, nscale_y))
                 data_np, (ascale_x, ascale_y) = trcalc.get_scaled_cutout_basic(
-                    data_np, 0, 0, wd-1, ht-1, nscale_x, nscale_y)
+                    data_np, 0, 0, wd-1, ht-1, nscale_x, nscale_y,
+                    logger=self.logger)
 
             # Rotate piece into our orientation, according to wcs
             rot_dx, rot_dy = xrot - xrot_ref, yrot - yrot_ref
@@ -508,7 +505,7 @@ class AstroImage(BaseImage):
 
             # Optomization for 180 rotations
             if (numpy.isclose(math.fabs(rot_dx), 180.0) or
-                numpy.isclose(math.fabs(rot_dy), 180.0)):
+                    numpy.isclose(math.fabs(rot_dy), 180.0)):
                 rotdata = trcalc.transform(data_np,
                                            flip_x=True, flip_y=True)
                 rot_dx = 0.0
@@ -522,14 +519,14 @@ class AstroImage(BaseImage):
                 self.logger.debug("rotating %s by %f deg" % (name, rot_deg))
                 rotdata = trcalc.rotate(rotdata, rot_deg,
                                         #rotctr_x=ctr_x, rotctr_y=ctr_y
-                                        )
+                                        logger=self.logger)
 
             # Flip X due to negative CDELT1
             if numpy.sign(cdelt1) != numpy.sign(cdelt1_ref):
                 flip_x = True
 
             # Flip Y due to negative CDELT2
-            if numpy.sign(cdelt2)  != numpy.sign(cdelt2_ref):
+            if numpy.sign(cdelt2) != numpy.sign(cdelt2_ref):
                 flip_y = True
 
             if flip_x or flip_y:
@@ -559,16 +556,17 @@ class AstroImage(BaseImage):
             xlo, xhi = x0 - ctr_x, x0 + wd - ctr_x
             ylo, yhi = y0 - ctr_y, y0 + ht - ctr_y
             assert (xhi - xlo == wd), \
-                   Exception("Width differential %d != %d" % (xhi - xlo, wd))
+                Exception("Width differential %d != %d" % (xhi - xlo, wd))
             assert (yhi - ylo == ht), \
-                   Exception("Height differential %d != %d" % (yhi - ylo, ht))
+                Exception("Height differential %d != %d" % (yhi - ylo, ht))
 
             mywd, myht = self.get_size()
             if xlo < 0 or xhi > mywd or ylo < 0 or yhi > myht:
                 if not allow_expand:
-                    raise Exception("New piece doesn't fit on image and allow_expand=False")
+                    raise Exception("New piece doesn't fit on image and "
+                                    "allow_expand=False")
 
-                #<-- Resize our data array to allow the new image
+                # <-- Resize our data array to allow the new image
 
                 # determine amount to pad expansion by
                 expand_x = max(int(expand_pad_deg / scale_x), 0)
@@ -596,7 +594,7 @@ class AstroImage(BaseImage):
                 new_area = new_wd * new_ht
                 expand_pct = new_area / old_area
                 if ((max_expand_pct is not None) and
-                    (expand_pct > max_expand_pct)):
+                        (expand_pct > max_expand_pct)):
                     raise Exception("New area exceeds current one by %.2f %%;"
                                     "increase max_expand_pct (%.2f) to allow" %
                                     (expand_pct*100, max_expand_pct))
@@ -605,7 +603,7 @@ class AstroImage(BaseImage):
                 new_data = numpy.zeros((new_ht, new_wd))
                 # place current data into new data
                 new_data[ny1_off:ny1_off+myht, nx1_off:nx1_off+mywd] = \
-                                               mydata
+                    mydata
                 self._data = new_data
                 mydata = new_data
 
@@ -623,7 +621,7 @@ class AstroImage(BaseImage):
                 else:
                     idx = (mydata[ylo:yhi, xlo:xhi, ...] == 0.0)
                     mydata[ylo:yhi, xlo:xhi, ...][idx] = \
-                                    rotdata[0:ht, 0:wd, ...][idx]
+                        rotdata[0:ht, 0:wd, ...][idx]
 
             except Exception as e:
                 self.logger.error("Error fitting tile: %s" % (str(e)))
@@ -657,7 +655,6 @@ class AstroImage(BaseImage):
         ra_lbl, dec_lbl = six.unichr(945), six.unichr(948)
 
         # Calculate WCS coords, if available
-        ts = time.time()
         try:
             if self.wcs is None:
                 self.logger.debug("No WCS for this image")
@@ -669,8 +666,7 @@ class AstroImage(BaseImage):
 
             elif self.wcs.coordsys == 'pixel':
                 args = [data_x, data_y] + self.revnaxis
-                x, y = self.wcs.pixtosystem(#(data_x, data_y),
-                    args, system=system, coords='data')
+                x, y = self.wcs.pixtosystem(args, system=system, coords='data')
                 ra_txt = "%+.3f" % (x)
                 dec_txt = "%+.3f" % (y)
                 ra_lbl, dec_lbl = "X", "Y"
@@ -678,13 +674,13 @@ class AstroImage(BaseImage):
             else:
                 args = [data_x, data_y] + self.revnaxis
 
-                lon_deg, lat_deg = self.wcs.pixtosystem(#(data_x, data_y),
+                lon_deg, lat_deg = self.wcs.pixtosystem(
                     args, system=system, coords='data')
 
                 if format == 'sexagesimal':
                     if system in ('galactic', 'ecliptic'):
                         sign, deg, min, sec = wcs.degToDms(lon_deg,
-                                                               isLatitude=False)
+                                                           isLatitude=False)
                         ra_txt = '+%03d:%02d:%06.3f' % (deg, min, sec)
                     else:
                         deg, min, sec = wcs.degToHms(lon_deg)
@@ -713,8 +709,7 @@ class AstroImage(BaseImage):
         except Exception as e:
             self.logger.warning("Bad coordinate conversion: %s" % (
                 str(e)))
-            ra_txt  = 'BAD WCS'
-            dec_txt = 'BAD WCS'
+            ra_txt = dec_txt = 'BAD WCS'
             try:
                 # log traceback, if possible
                 (type_, value_, tb) = sys.exc_info()
@@ -724,8 +719,6 @@ class AstroImage(BaseImage):
                 tb_str = "Traceback information unavailable."
                 self.logger.error(tb_str)
 
-        te = time.time() - ts
-        #print "time elapsed: %.4f" % te
         info = Bunch.Bunch(itype='astro', data_x=data_x, data_y=data_y,
                            x=data_x, y=data_y,
                            ra_txt=ra_txt, dec_txt=dec_txt,
@@ -733,4 +726,4 @@ class AstroImage(BaseImage):
                            value=value)
         return info
 
-#END
+# END
