@@ -5,11 +5,14 @@
 # Please see the file LICENSE.txt for details.
 
 import math
+import os.path
 import itertools
+import numpy as np
 
 from ginga.misc import Bunch, Settings, Callback
 from ginga import AutoCuts, trcalc
 from ginga import cmap, imap
+from ginga.util.paths import icondir
 
 
 class ImageViewBindings(object):
@@ -25,7 +28,6 @@ class ImageViewBindings(object):
 
         self.canpan = False
         self.canzoom = False
-        self._ispanning = False
         self.cancut = False
         self.cancmap = False
         self.canflip = False
@@ -55,15 +57,20 @@ class ImageViewBindings(object):
             # name, attr pairs
             pan='canpan', zoom='canzoom', cuts='cancut', cmap='cancmap',
             flip='canflip', rotate='canrotate')
+        self.cursor_map = {}
 
     def initialize_settings(self, settings):
         settings.addSettings(
             # You should rarely have to change these.
             btn_nobtn = 0x0,
             btn_left  = 0x1,
-            btn_middle= 0x2,
+            btn_middle = 0x2,
             btn_right = 0x4,
 
+            # define our cursors
+            ## cur_pick = 'thinCrossCursor',
+            ## cur_pan = 'openHandCursor',
+
             # Set up our standard modifiers
             mod_shift = ['shift_l', 'shift_r'],
             mod_ctrl = ['control_l', 'control_r'],
@@ -76,8 +83,8 @@ class ImageViewBindings(object):
             dmod_dist = ['d', None, None],
             dmod_contrast = ['t', None, None],
             dmod_rotate = ['r', None, None],
-            dmod_pan = ['q', None, None],
-            dmod_freepan = ['w', None, None],
+            dmod_pan = ['q', None, 'pan'],
+            dmod_freepan = ['w', None, 'pan'],
 
             default_mode_type = 'oneshot',
             default_lock_mode_type = 'softlock',
@@ -112,13 +119,14 @@ class ImageViewBindings(object):
             kp_autocenter_override = ['/', 'pan+/'],
             kp_contrast_restore = ['T', 'contrast+T'],
             kp_cmap_reset = ['Y', 'cmap+Y'],
+            kp_cmap_restore = ['cmap+r'],
             kp_cmap_invert = ['I', 'cmap+I'],
             kp_cmap_prev = ['cmap+up', 'cmap+b'],
             kp_cmap_next = ['cmap+down', 'cmap+n'],
             kp_toggle_cbar = ['cmap+c'],
             kp_imap_reset = ['cmap+i'],
-            kp_imap_prev = ['cmap+k'],
-            kp_imap_next = ['cmap+l'],
+            kp_imap_prev = ['cmap+left', 'cmap+j'],
+            kp_imap_next = ['cmap+right', 'cmap+k'],
             kp_flip_x = ['[', '{', 'rotate+[', 'rotate+{'],
             kp_flip_y = [']', '}', 'rotate+]', 'rotate+}'],
             kp_swap_xy = ['backslash', '|', 'rotate+backslash', 'rotate+|'],
@@ -138,12 +146,13 @@ class ImageViewBindings(object):
             key_pan_pct = 0.666667,
 
             # SCROLLING/WHEEL
-            sc_pan = ['shift+scroll'],
+            sc_pan = ['ctrl+scroll'],
             sc_pan_fine = ['pan+shift+scroll'],
             sc_pan_coarse = ['pan+ctrl+scroll'],
             sc_zoom = ['scroll'],
-            sc_zoom_fine = ['ctrl+scroll'],
+            sc_zoom_fine = [],
             sc_zoom_coarse = [],
+            sc_zoom_origin = ['shift+scroll', 'freepan+scroll'],
             sc_cuts_fine = ['cuts+ctrl+scroll'],
             sc_cuts_coarse = ['cuts+scroll'],
             sc_cuts_alg = [],
@@ -216,6 +225,8 @@ class ImageViewBindings(object):
         bindmap.clear_button_map()
         bindmap.clear_event_map()
 
+        bindmap.add_callback('mode-set', self.mode_set_cb, viewer)
+
         # Set up bindings
         self.setup_settings_events(viewer, bindmap)
 
@@ -223,6 +234,10 @@ class ImageViewBindings(object):
         bindmap = viewer.get_bindmap()
         bindmap.set_mode(name, mode_type=mode_type)
 
+    def mode_set_cb(self, bm, mode, mode_type, viewer):
+        cursor_name = self.cursor_map.get(mode, 'pick')
+        viewer.switch_cursor(cursor_name)
+
     def parse_combo(self, combo, modes_set, modifiers_set, pfx):
         """
         Parse a string into a mode, a set of modifiers and a trigger.
@@ -281,15 +296,21 @@ class ImageViewBindings(object):
                     keyname = combo
                     bindmap.add_modifier(keyname, modname)
 
+            elif name.startswith('cur_'):
+                curname = name[4:]
+                self.add_cursor(viewer, curname, value)
+
             elif name.startswith('btn_'):
                 btnname = name[4:]
                 bindmap.map_button(value, btnname)
 
             elif name.startswith('dmod_'):
                 mode_name = name[5:]
-                keyname, mode_type, msg = value
+                keyname, mode_type, curname = value
                 bindmap.add_mode(keyname, mode_name, mode_type=mode_type,
-                                     msg=msg)
+                                     msg=None)
+                if curname is not None:
+                    self.cursor_map[mode_name] = curname
 
         modes_set = bindmap.get_modes()
         modifiers_set = bindmap.get_modifiers()
@@ -354,9 +375,14 @@ class ImageViewBindings(object):
     def reset(self, viewer):
         bindmap = viewer.get_bindmap()
         bindmap.reset_mode(viewer)
-        self.pan_stop(viewer)
         viewer.onscreen_message(None)
 
+    def add_cursor(self, viewer, curname, curpath):
+        if not curpath.startswith('/'):
+            curpath = os.path.join(icondir, curpath)
+        cursor = viewer.make_cursor(curpath, 8, 8)
+        viewer.define_cursor(curname, cursor)
+
     #####  ENABLERS #####
     # These methods are a quick way to enable or disable certain user
     # interface features in a ImageView window
@@ -481,30 +507,129 @@ class ImageViewBindings(object):
             amt /= 10.0
         return amt
 
-    def pan_lr(self, viewer, pct_vw, sign, msg=False):
-        # calc amt of data shown in X relative to data size of image
-        wd, ht = viewer.get_data_size()
-        prect = viewer.get_pan_rect()
-        dwd = abs(prect[2][0] - prect[0][0])
-        # shift the pan by percentage of viewer width
-        amt = sign * dwd * pct_vw
-        pan_x, pan_y = viewer.get_pan(coord='data')
-        pan_x += amt
+    def calc_pan_pct(self, viewer, pad=0):
+        """Calculate values for vertical/horizontal panning by percentages
+        from the current pan position.
+        """
+        limits = viewer.get_limits()
 
-        self._panset(viewer, pan_x, pan_y, msg=msg)
+        tr = viewer.tform['data_to_scrollbar']
+
+        # calculate the corners of the entire image in unscaled cartesian
+        mxwd, mxht = limits[1]
+        mxwd, mxht = mxwd + pad, mxht + pad
+        mnwd, mnht = limits[0]
+        mnwd, mnht = mnwd - pad, mnht - pad
+
+        arr = np.array([(mnwd, mnht), (mxwd, mnht),
+                        (mxwd, mxht), (mnwd, mxht)],
+                       dtype=np.float)
+        x, y = tr.to_(arr.T[0], arr.T[1])
+
+        rx1, rx2 = np.min(x), np.max(x)
+        ry1, ry2 = np.min(y), np.max(y)
+
+        rect = viewer.get_pan_rect()
+        arr = np.array(rect, dtype=np.float)
+        x, y = tr.to_(arr.T[0], arr.T[1])
+
+        qx1, qx2 = np.min(x), np.max(x)
+        qy1, qy2 = np.min(y), np.max(y)
+
+        qx1, qx2 = max(rx1, qx1), min(rx2, qx2)
+        qy1, qy2 = max(ry1, qy1), min(ry2, qy2)
+
+        # this is the range of X and Y of the entire image
+        # in the viewer (unscaled)
+        rng_x, rng_y = abs(rx2 - rx1), abs(ry2 - ry1)
+
+        # this is the *visually shown* range of X and Y
+        abs_x, abs_y = abs(qx2 - qx1), abs(qy2 - qy1)
+
+        # calculate the width of the slider arms as a ratio
+        xthm_pct = max(0.0, min(abs_x / (rx2 - rx1), 1.0))
+        ythm_pct = max(0.0, min(abs_y / (ry2 - ry1), 1.0))
+
+        # calculate the pan position as a ratio
+        pct_x = min(max(0.0, abs(0.0 - rx1) / rng_x), 1.0)
+        pct_y = min(max(0.0, abs(0.0 - ry1) / rng_y), 1.0)
+
+        return Bunch.Bunch(rng_x=rng_x, rng_y=rng_y, vis_x=abs_x, vis_y=abs_y,
+                           thm_pct_x=xthm_pct, thm_pct_y=ythm_pct,
+                           pan_pct_x=pct_x, pan_pct_y=pct_y)
+
+    def pan_by_pct(self, viewer, pct_x, pct_y, pad=0):
+        """Called when the scroll bars are adjusted by the user.
+        """
+        limits = viewer.get_limits()
+
+        tr = viewer.tform['data_to_scrollbar']
+
+        mxwd, mxht = limits[1]
+        mxwd, mxht = mxwd + pad, mxht + pad
+        mnwd, mnht = limits[0]
+        mnwd, mnht = mnwd - pad, mnht - pad
+
+        arr = np.array([(mnwd, mnht), (mxwd, mnht),
+                        (mxwd, mxht), (mnwd, mxht)],
+                       dtype=np.float)
+        x, y = tr.to_(arr.T[0], arr.T[1])
+
+        rx1, rx2 = np.min(x), np.max(x)
+        ry1, ry2 = np.min(y), np.max(y)
+
+        crd_x = rx1 + (pct_x * (rx2 - rx1))
+        crd_y = ry1 + (pct_y * (ry2 - ry1))
+
+        pan_x, pan_y = tr.from_(crd_x, crd_y)
+        self.logger.debug("crd=%f,%f pan=%f,%f" % (
+            crd_x, crd_y, pan_x, pan_y))
+
+        viewer.panset_xy(pan_x, pan_y)
+
+    def pan_omni(self, viewer, direction, amount, msg=False):
+        # calculate current pan pct
+        res = self.calc_pan_pct(viewer, pad=0)
+
+        ang_rad = math.radians(90.0 - direction)
+        amt_x = math.cos(ang_rad) * amount
+        amt_y = math.sin(ang_rad) * amount
+
+        # modify the pct, as per the params
+        pct_page_x = res.vis_x / res.rng_x
+        amt_x = amt_x * pct_page_x
+        pct_page_y = res.vis_y / res.rng_y
+        amt_y = amt_y * pct_page_y
+
+        pct_x = res.pan_pct_x + amt_x
+        pct_y = res.pan_pct_y + amt_y
+
+        # update the pan position by pct
+        self.pan_by_pct(viewer, pct_x, pct_y)
+
+    def pan_lr(self, viewer, pct_vw, sign, msg=False):
+        # calculate current pan pct
+        res = self.calc_pan_pct(viewer, pad=0)
+
+        pct_page = res.vis_x / res.rng_x
+        # modify the pct, as per the params
+        amt = sign * pct_vw * pct_page
+        pct_x = res.pan_pct_x + amt
+
+        # update the pan position by pct
+        self.pan_by_pct(viewer, pct_x, res.pan_pct_y)
 
     def pan_ud(self, viewer, pct_vh, sign, msg=False):
-        # calc amt of data shown in X relative to data size of image
-        wd, ht = viewer.get_data_size()
-        prect = viewer.get_pan_rect()
-        dht = abs(prect[2][1] - prect[0][1])
-        # shift the pan by that percentage
-        #pct = 1.0 + sign * dht / float(ht) * (1.0 - pct_vh)
-        amt = sign * dht * pct_vh
-        pan_x, pan_y = viewer.get_pan(coord='data')
-        pan_y += amt
+        # calculate current pan pct
+        res = self.calc_pan_pct(viewer, pad=0)
 
-        self._panset(viewer, pan_x, pan_y, msg=msg)
+        pct_page = res.vis_y / res.rng_y
+        # modify the pct, as per the params
+        amt = sign * pct_vh * pct_page
+        pct_y = res.pan_pct_y + amt
+
+        # update the pan position by pct
+        self.pan_by_pct(viewer, res.pan_pct_x, pct_y)
 
     def get_direction(self, direction, rev=False):
         """
@@ -648,7 +773,9 @@ class ImageViewBindings(object):
         elif direction == 'down':
             mult = 1.0 / factor
         scale_x, scale_y = scale_x * mult, scale_y * mult
-        viewer.scale_to(scale_x, scale_y)
+        scale = max(scale_x, scale_y)
+        #viewer.scale_to(scale_x, scale_y)
+        viewer.scale_to(scale, scale)
         if msg:
             viewer.onscreen_message(viewer.get_scale_text(),
                                        delay=0.4)
@@ -660,7 +787,6 @@ class ImageViewBindings(object):
         direction = 0.0
         if delta < 0.0:
             direction = 180.0
-        #print("factor=%f direction=%f" % (factor, direction))
         self._start_x = x
         self._scale_image(viewer, direction, factor, msg=msg)
 
@@ -710,34 +836,6 @@ class ImageViewBindings(object):
                 viewer.onscreen_message("Autocuts alg: %s" % (algname),
                                         delay=1.0)
 
-    def show_cbar(self, viewer):
-        canvas = viewer.private_canvas
-        # canvas already has a color bar?
-        objs = list(canvas.get_objects_by_kinds(('colorbar', 'drawablecolorbar')))
-        if len(objs) == 0:
-            # no color bar present
-            Cbar = canvas.get_draw_class('colorbar')
-            cbar = Cbar()
-            canvas.add(cbar, tag='__cbar')
-
-    def remove_cbar(self, viewer):
-        canvas = viewer.private_canvas
-        try:
-            # canvas already has a color bar?
-            canvas.delete_object_by_tag('__cbar')
-
-        except KeyError:
-            pass
-
-    def toggle_cbar(self, viewer):
-        canvas = viewer.private_canvas
-        try:
-            obj = canvas.get_object_by_tag('__cbar')
-            self.remove_cbar(viewer)
-
-        except KeyError:
-            self.show_cbar(viewer)
-
     def _cycle_cmap(self, viewer, msg, direction='down'):
         if self.cancmap:
             msg = self.settings.get('msg_cmap', msg)
@@ -867,31 +965,31 @@ class ImageViewBindings(object):
                 degn, str(xflip)), delay=1.0)
 
     def to_default_mode(self, viewer):
-        self._ispanning = False
-        viewer.switch_cursor('pick')
+        self.reset(viewer)
 
     def pan_start(self, viewer, ptype=1):
-        # If already panning then ignore multiple keystrokes
-        if self._ispanning:
-            return
         self._pantype = ptype
-        viewer.switch_cursor('pan')
-        self._ispanning = True
 
     def pan_set_origin(self, viewer, win_x, win_y, data_x, data_y):
         self._start_x, self._start_y = viewer.window_to_offset(win_x, win_y)
         self._start_panx, self._start_pany = viewer.get_pan()
 
     def pan_stop(self, viewer):
-        self._ispanning = False
         self._start_x = None
         self._pantype = 1
-        self.to_default_mode(viewer)
+
+    def restore_contrast(self, viewer, msg=True):
+        msg = self.settings.get('msg_cmap', msg)
+        rgbmap = viewer.get_rgbmap()
+        rgbmap.reset_sarr()
+        if msg:
+            viewer.onscreen_message("Restored contrast", delay=0.5)
+        return True
 
     def restore_colormap(self, viewer, msg=True):
         msg = self.settings.get('msg_cmap', msg)
         rgbmap = viewer.get_rgbmap()
-        rgbmap.reset_sarr()
+        rgbmap.restore_cmap()
         if msg:
             viewer.onscreen_message("Restored color map", delay=0.5)
         return True
@@ -1110,7 +1208,7 @@ class ImageViewBindings(object):
     def kp_contrast_restore(self, viewer, event, data_x, data_y, msg=True):
         if self.cancmap:
             msg = self.settings.get('msg_cmap', msg)
-            self.restore_colormap(viewer, msg=msg)
+            self.restore_contrast(viewer, msg=msg)
         return True
 
     def kp_flip_x(self, viewer, event, data_x, data_y, msg=True):
@@ -1172,6 +1270,10 @@ class ImageViewBindings(object):
         self._reset_cmap(viewer, msg)
         return True
 
+    def kp_cmap_restore(self, viewer, event, data_x, data_y, msg=True):
+        self.restore_colormap(viewer, msg)
+        return True
+
     def kp_cmap_invert(self, viewer, event, data_x, data_y, msg=True):
         self._invert_cmap(viewer, msg)
         return True
@@ -1185,7 +1287,11 @@ class ImageViewBindings(object):
         return True
 
     def kp_toggle_cbar(self, viewer, event, data_x, data_y, msg=True):
-        self.toggle_cbar(viewer)
+        canvas = viewer.get_private_canvas()
+        # canvas already has a color bar?
+        objs = list(canvas.get_objects_by_kinds(('colorbar', 'drawablecolorbar')))
+        tf = (len(objs) == 0)
+        viewer.show_color_bar(tf)
         return True
 
     def kp_imap_reset(self, viewer, event, data_x, data_y, msg=True):
@@ -1290,8 +1396,10 @@ class ImageViewBindings(object):
             viewer.onscreen_message(None)
         return True
 
-    def _scale_adjust(self, factor, zoom_accel, max_limit=None):
-        amount = factor - ((factor - 1.0) * (1.0 - zoom_accel))
+    def _scale_adjust(self, factor, event_amt, zoom_accel, max_limit=None):
+        # adjust scale by factor, amount encoded in event and zoom acceleration value
+        amount = factor - ((factor - 1.0) * (1.0 - min(event_amt, 15.0) / 15.0) *
+                           zoom_accel)
         amount = max(1.000000001, amount)
         if max_limit is not None:
             amount = min(amount, max_limit)
@@ -1312,7 +1420,7 @@ class ImageViewBindings(object):
             if self.settings.get('scroll_zoom_direct_scale', True):
                 zoom_accel = self.settings.get('scroll_zoom_acceleration', 1.0)
                 # change scale by 100%
-                amount = self._scale_adjust(2.0, zoom_accel, max_limit=4.0)
+                amount = self._scale_adjust(2.0, 15.0, zoom_accel, max_limit=4.0)
                 self._scale_image(viewer, 0.0, amount, msg=msg)
             else:
                 viewer.zoom_in()
@@ -1339,7 +1447,7 @@ class ImageViewBindings(object):
             if self.settings.get('scroll_zoom_direct_scale', True):
                 zoom_accel = self.settings.get('scroll_zoom_acceleration', 1.0)
                 # change scale by 100%
-                amount = self._scale_adjust(2.0, zoom_accel, max_limit=4.0)
+                amount = self._scale_adjust(2.0, 15.0, zoom_accel, max_limit=4.0)
                 self._scale_image(viewer, 180.0, amount, msg=msg)
             else:
                 viewer.zoom_out()
@@ -1411,11 +1519,11 @@ class ImageViewBindings(object):
 
 
     def ms_contrast_restore(self, viewer, event, data_x, data_y, msg=True):
-        """An interactive way to restore the colormap settings after
+        """An interactive way to restore the colormap contrast settings after
         a warp operation.
         """
         if self.cancmap and (event.state == 'down'):
-            self.restore_colormap(viewer, msg=msg)
+            self.restore_contrast(viewer, msg=msg)
         return True
 
 
@@ -1445,11 +1553,10 @@ class ImageViewBindings(object):
 
     def ms_cmap_restore(self, viewer, event, data_x, data_y, msg=True):
         """An interactive way to restore the colormap settings after
-        a warp operation.
+        a rotate or invert operation.
         """
         if self.cancmap and (event.state == 'down'):
-            rgbmap = viewer.get_rgbmap()
-            rgbmap.restore_cmap()
+            self.restore_colormap(viewer, msg)
         return True
 
 
@@ -1595,53 +1702,94 @@ class ImageViewBindings(object):
             self._cycle_cuts_alg(viewer, msg, direction=direction)
         return True
 
-    def sc_zoom(self, viewer, event, msg=True):
-        """Interactively zoom the image by scrolling motion.
-        This zooms by the zoom steps configured under Preferences.
-        """
+    def zoom_step(self, viewer, event, msg=True, origin=None, adjust=1.5):
+        with viewer.suppress_redraw:
+
+            if origin is not None:
+                # get cartesian canvas coords of data item under cursor
+                data_x, data_y = origin
+                off_x, off_y = viewer.data_to_offset(data_x, data_y)
+                # set the pan position to the data item
+                viewer.set_pan(data_x, data_y)
+
+            # scale by the desired means
+            if self.settings.get('scroll_zoom_direct_scale', True):
+                zoom_accel = self.settings.get('scroll_zoom_acceleration', 1.0)
+                # change scale by 50%
+                amount = self._scale_adjust(adjust, event.amount, zoom_accel,
+                                            max_limit=4.0)
+                self._scale_image(viewer, event.direction, amount, msg=msg)
+
+            else:
+                rev = self.settings.get('zoom_scroll_reverse', False)
+                direction = self.get_direction(event.direction, rev=rev)
+
+                if direction == 'up':
+                    viewer.zoom_in()
+
+                elif direction == 'down':
+                    viewer.zoom_out()
+
+                if msg:
+                    viewer.onscreen_message(viewer.get_scale_text(),
+                                            delay=0.4)
+
+            if origin is not None:
+                # now adjust the pan position to keep the offset
+                data_x2, data_y2 = viewer.offset_to_data(off_x, off_y)
+                dx, dy = data_x2 - data_x , data_y2 - data_y
+                viewer.panset_xy(data_x - dx, data_y - dy)
+
+    def _sc_zoom(self, viewer, event, msg=True, origin=None):
         if not self.canzoom:
             return True
 
         msg = self.settings.get('msg_zoom', msg)
 
-        if self.settings.get('scroll_zoom_direct_scale', True):
-            zoom_accel = self.settings.get('scroll_zoom_acceleration', 1.0)
-            # change scale by 50%
-            amount = self._scale_adjust(1.5, zoom_accel, max_limit=4.0)
-            self._scale_image(viewer, event.direction, amount, msg=msg)
+        self.zoom_step(viewer, event, msg=msg, origin=origin,
+                       adjust=1.5)
+        return True
 
-        else:
-            rev = self.settings.get('zoom_scroll_reverse', False)
-            direction = self.get_direction(event.direction, rev=rev)
-            if direction == 'up':
-                viewer.zoom_in()
-            elif direction == 'down':
-                viewer.zoom_out()
-                if msg:
-                    viewer.onscreen_message(viewer.get_scale_text(),
-                                            delay=0.4)
+    def sc_zoom(self, viewer, event, msg=True):
+        """Interactively zoom the image by scrolling motion.
+        This zooms by the zoom steps configured under Preferences.
+        """
+        self._sc_zoom(viewer, event, msg=msg, origin=None)
+        return True
+
+    def sc_zoom_origin(self, viewer, event, msg=True):
+        """Like sc_zoom(), but pans the image as well to keep the
+        coordinate under the cursor in that same position relative
+        to the window.
+        """
+        origin = (event.data_x, event.data_y)
+        self._sc_zoom(viewer, event, msg=msg, origin=origin)
         return True
 
     def sc_zoom_coarse(self, viewer, event, msg=True):
         """Interactively zoom the image by scrolling motion.
         This zooms by adjusting the scale in x and y coarsely.
         """
-        if self.canzoom:
-            zoom_accel = self.settings.get('scroll_zoom_acceleration', 1.0)
-            # change scale by 20%
-            amount = self._scale_adjust(1.2, zoom_accel, max_limit=4.0)
-            self._scale_image(viewer, event.direction, amount, msg=msg)
+        if not self.canzoom:
+            return True
+
+        zoom_accel = self.settings.get('scroll_zoom_acceleration', 1.0)
+        # change scale by 20%
+        amount = self._scale_adjust(1.2, event.amount, zoom_accel, max_limit=4.0)
+        self._scale_image(viewer, event.direction, amount, msg=msg)
         return True
 
     def sc_zoom_fine(self, viewer, event, msg=True):
         """Interactively zoom the image by scrolling motion.
         This zooms by adjusting the scale in x and y coarsely.
         """
-        if self.canzoom:
-            zoom_accel = self.settings.get('scroll_zoom_acceleration', 1.0)
-            # change scale by 5%
-            amount = self._scale_adjust(1.05, zoom_accel, max_limit=4.0)
-            self._scale_image(viewer, event.direction, amount, msg=msg)
+        if not self.canzoom:
+            return True
+
+        zoom_accel = self.settings.get('scroll_zoom_acceleration', 1.0)
+        # change scale by 5%
+        amount = self._scale_adjust(1.05, event.amount, zoom_accel, max_limit=4.0)
+        self._scale_image(viewer, event.direction, amount, msg=msg)
         return True
 
     def sc_pan(self, viewer, event, msg=True):
@@ -1653,40 +1801,17 @@ class ImageViewBindings(object):
         # User has "Pan Reverse" preference set?
         rev = self.settings.get('pan_reverse', False)
         direction = event.direction
+
         if rev:
             direction = math.fmod(direction + 180.0, 360.0)
 
         pan_accel = self.settings.get('scroll_pan_acceleration', 1.0)
-        num_degrees = event.amount * pan_accel
-        ang_rad = math.radians(90.0 - direction)
+        # Internal factor to adjust the panning speed so that user-adjustable
+        # scroll_pan_acceleration is normalized to 1.0 for "normal" speed
+        scr_pan_adj_factor = 1.4142135623730951
+        amount = (event.amount * scr_pan_adj_factor * pan_accel) / 360.0
 
-        # Calculate distance of pan amount, based on current scale
-        wd, ht = viewer.get_data_size()
-        # pageSize = min(wd, ht)
-        ((x0, y0), (x1, y1), (x2, y2), (x3, y3)) = viewer.get_pan_rect()
-        page_size = min(abs(x2 - x0), abs(y2 - y0))
-        distance = (num_degrees / 360.0) * page_size
-        self.logger.debug("angle=%f ang_rad=%f distance=%f" % (
-            direction, ang_rad, distance))
-
-        # Calculate new pan position
-        pan_x, pan_y = viewer.get_pan()
-        new_x = pan_x + math.cos(ang_rad) * distance
-        new_y = pan_y + math.sin(ang_rad) * distance
-
-        # cap pan position
-        new_x = min(max(new_x, 0.0), wd)
-        new_y = min(max(new_y, 0.0), ht)
-
-        # Because pan position is reported +0.5
-        #new_x, new_y = new_x - 0.5, new_y - 0.5
-        #print "data x,y=%f,%f   new x, y=%f,%f" % (pan_x, pan_y, new_x, new_y)
-
-        viewer.panset_xy(new_x, new_y)
-
-        # For checking result
-        #pan_x, pan_y = viewer.get_pan()
-        #print "new pan x,y=%f, %f" % (pan_x, pan_y)
+        self.pan_omni(viewer, direction, amount)
         return True
 
     def sc_pan_coarse(self, viewer, event, msg=True):
@@ -1727,8 +1852,15 @@ class ImageViewBindings(object):
             if self.canzoom and ('zoom' in pinch_actions):
                 scale_accel = self.settings.get('pinch_zoom_acceleration', 1.0)
                 scale = scale * scale_accel
-                scale_x, scale_y = (self._start_scale_x * scale,
-                                    self._start_scale_y * scale)
+                # NOTE: scale reported by Qt (the only toolkit that really gives
+                # us a pinch gesture right now) the scale reports are iterative
+                # so it makes more sense to keep updating the scale than to base
+                # the scale on the original scale captured when the gesture starts
+                ## scale_x, scale_y = (self._start_scale_x * scale,
+                ##                     self._start_scale_y * scale)
+                scale_x, scale_y = viewer.get_scale_xy()
+                scale_x, scale_y = scale_x * scale, scale_y * scale
+
                 viewer.scale_to(scale_x, scale_y)
                 msg_str = viewer.get_scale_text()
                 msg = self.settings.get('msg_zoom', True)
@@ -1996,8 +2128,6 @@ class BindingMapper(Callback.Callbacks):
         if not has_focus:
             # fixes a problem with not receiving key release events when the
             # window loses focus
-            # TODO: does this need to force a key release callback if there
-            # are any keys held?
             self._modifiers = frozenset([])
         return True
 
