@@ -1,9 +1,6 @@
 #
 # ColorDist.py -- Color Distribution algorithms
-# 
-# Eric Jeschke (eric@naoj.org)
 #
-# Copyright (c) Eric R. Jeschke.  All rights reserved.
 # This is open-source software licensed under a BSD license.
 # Please see the file LICENSE.txt for details.
 #
@@ -11,7 +8,7 @@
 These algorithms are modeled after the ones described for ds9 here:
 
     http://ds9.si.edu/doc/ref/how.html
-    
+
 """
 import math
 import numpy
@@ -39,10 +36,10 @@ class ColorDistBase(object):
         idx = idx.clip(0, self.hashsize-1)
         arr = self.hash[idx]
         return arr
-        
+
     def get_hash_size(self):
         return self.hashsize
-    
+
     def set_hash_size(self, size):
         assert (size >= self.colorlen) and (size <= self.maxhashsize), \
                ColorDistError("Bad hash size!")
@@ -66,7 +63,7 @@ class LinearDist(ColorDistBase):
     y = x
         where x in (0..1)
     """
-    
+
     def __init__(self, hashsize, colorlen=None):
         super(LinearDist, self).__init__(hashsize, colorlen=colorlen)
 
@@ -75,9 +72,9 @@ class LinearDist(ColorDistBase):
         # normalize to color range
         l = base * (self.colorlen - 1)
         self.hash = l.astype(numpy.uint)
-        
+
         self.check_hash()
-        
+
     def get_dist_pct(self, pct):
         val = min(max(float(pct), 0.0), 1.0)
         return val
@@ -91,7 +88,7 @@ class LogDist(ColorDistBase):
     y = log(a*x + 1) / log(a)
         where x in (0..1)
     """
-    
+
     def __init__(self, hashsize, colorlen=None, exp=1000.0):
         self.exp = exp
         super(LogDist, self).__init__(hashsize, colorlen=colorlen)
@@ -103,7 +100,7 @@ class LogDist(ColorDistBase):
         # normalize to color range
         l = base * (self.colorlen - 1)
         self.hash = l.astype(numpy.uint)
-        
+
         self.check_hash()
 
     def get_dist_pct(self, pct):
@@ -120,7 +117,7 @@ class PowerDist(ColorDistBase):
     y = ((a ** x) - 1) / a
         where x in (0..1)
     """
-    
+
     def __init__(self, hashsize, colorlen=None, exp=1000.0):
         self.exp = exp
         super(PowerDist, self).__init__(hashsize, colorlen=colorlen)
@@ -132,7 +129,7 @@ class PowerDist(ColorDistBase):
         # normalize to color range
         l = base * (self.colorlen - 1)
         self.hash = l.astype(numpy.uint)
-        
+
         self.check_hash()
 
     def get_dist_pct(self, pct):
@@ -149,7 +146,7 @@ class SqrtDist(ColorDistBase):
     y = sqrt(x)
         where x in (0..1)
     """
-    
+
     def __init__(self, hashsize, colorlen=None):
         super(SqrtDist, self).__init__(hashsize, colorlen=colorlen)
 
@@ -160,7 +157,7 @@ class SqrtDist(ColorDistBase):
         # normalize to color range
         l = base * (self.colorlen - 1)
         self.hash = l.astype(numpy.uint)
-        
+
         self.check_hash()
 
     def get_dist_pct(self, pct):
@@ -177,7 +174,7 @@ class SquaredDist(ColorDistBase):
     y = x ** 2
         where x in (0..1)
     """
-    
+
     def __init__(self, hashsize, colorlen=None):
         super(SquaredDist, self).__init__(hashsize, colorlen=colorlen)
 
@@ -187,7 +184,7 @@ class SquaredDist(ColorDistBase):
         # normalize to color range
         l = base * (self.colorlen - 1)
         self.hash = l.astype(numpy.uint)
-        
+
         self.check_hash()
 
     def get_dist_pct(self, pct):
@@ -204,7 +201,7 @@ class AsinhDist(ColorDistBase):
     y = asinh(nonlinearity * x) / factor
         where x in (0..1)
     """
-    
+
     def __init__(self, hashsize, colorlen=None, factor=10.0,
                  nonlinearity=3.0):
         self.factor = factor
@@ -236,7 +233,7 @@ class SinhDist(ColorDistBase):
     y = sinh(factor * x) / nonlinearity
         where x in (0..1)
     """
-    
+
     def __init__(self, hashsize, colorlen=None, factor=3.0,
                  nonlinearity=10.0):
         self.factor = factor
@@ -268,7 +265,7 @@ class HistogramEqualizationDist(ColorDistBase):
     The histogram equalization distribution function distributes colors
     based on the frequency of each data value.
     """
-    
+
     def __init__(self, hashsize, colorlen=None):
         super(HistogramEqualizationDist, self).__init__(hashsize,
                                                          colorlen=colorlen)
@@ -286,7 +283,7 @@ class HistogramEqualizationDist(ColorDistBase):
         # NOTE: data could be assumed to be in the range 0..hashsize-1
         # at this point but clip as a precaution
         idx = idx.clip(0, self.hashsize-1)
-        
+
         #get image histogram
         hist, bins = numpy.histogram(idx.flatten(),
                                      self.hashsize, density=False)
@@ -300,7 +297,7 @@ class HistogramEqualizationDist(ColorDistBase):
 
         arr = self.hash[idx]
         return arr
-        
+
     def get_dist_pct(self, pct):
         # TODO: this is wrong but we need a way to invert the hash
         return pct
@@ -320,19 +317,23 @@ distributions = {
     'sinh': SinhDist,
     'histeq': HistogramEqualizationDist,
     }
-    
+
 def add_dist(name, distClass):
     global distributions
     distributions[name.lower()] = distClass
-    
+
 def get_dist_names():
-    #return distributions.keys()
-    return ['linear', 'log', 'power', 'sqrt', 'squared', 'asinh', 'sinh',
-            'histeq']
-    
+    a_names = set(distributions.keys())
+    std_names = ['linear', 'log', 'power', 'sqrt', 'squared', 'asinh', 'sinh',
+                 'histeq']
+    rest = a_names - set(std_names)
+    if len(rest) > 0:
+        std_names = std_names + list(rest)
+    return std_names
+
 def get_dist(name):
     if not name in distributions:
         raise ColorDistError("Invalid distribution algorithm '%s'" % (name))
     return distributions[name]
-    
+
 #END
