@@ -60,7 +60,8 @@ class CanvasObjectBase(Callback.Callbacks):
         self.viewer = None
 
         # For callbacks
-        for name in ('edited', 'pick-down', 'pick-move', 'pick-up'):
+        for name in ('edited', 'pick-down', 'pick-move', 'pick-up',
+                     'pick-hover', 'pick-enter', 'pick-leave'):
             self.enable_callback(name)
 
     def initialize(self, canvas, viewer, logger):
@@ -104,8 +105,11 @@ class CanvasObjectBase(Callback.Callbacks):
     def use_coordmap(self, mapobj):
         self.crdmap = mapobj
 
-    def canvascoords(self, viewer, data_x, data_y, center=True):
-        return viewer.get_canvas_xy(data_x, data_y, center=center)
+    def canvascoords(self, viewer, data_x, data_y, center=None):
+        if center is not None:
+            self.logger.warn("`center` keyword is ignored and will be deprecated")
+
+        return viewer.get_canvas_xy(data_x, data_y)
 
     def is_compound(self):
         return False
