@@ -48,6 +48,7 @@ class DrawingMixin(object):
         self._edit_obj = None
         self._edit_status = False
         self._edit_detail = {}
+        self._pick_cur_obj = None
 
         # For modes
         self._mode = 'draw'
@@ -63,6 +64,7 @@ class DrawingMixin(object):
                            poly_delete=self.edit_poly_delete)
         self.add_draw_mode('pick', down=self.pick_start,
                            move=self.pick_motion, up=self.pick_stop,
+                           hover=self.pick_hover,
                            poly_add=self.edit_poly_add,
                            poly_delete=self.edit_poly_delete)
 
@@ -113,6 +115,7 @@ class DrawingMixin(object):
         canvas.add_callback('cursor-down', self._draw_op, 'down', viewer)
         canvas.add_callback('cursor-move', self._draw_op, 'move', viewer)
         canvas.add_callback('cursor-up', self._draw_op, 'up', viewer)
+        canvas.set_callback('none-move', self._draw_op, 'hover', viewer)
 
     ##### MODE LOGIC #####
 
@@ -170,24 +173,25 @@ class DrawingMixin(object):
 
     ##### DRAWING LOGIC #####
 
-    def _draw_update(self, data_x, data_y, cxt):
+    def _draw_update(self, data_x, data_y, cxt, force_update=False):
 
-        ## self.logger.debug("drawing a '%s' x,y=%f,%f" % (
-        ##     self.t_drawtype, data_x, data_y))
-
-        klass = self.draw_dict[self.t_drawtype]
         obj = None
 
         # update the context with current position
         x, y = cxt.crdmap.data_to(data_x, data_y)
         cxt.setvals(x=x, y=y, data_x=data_x, data_y=data_y)
 
-        obj = klass.idraw(self, cxt)
+        draw_class = cxt.draw_class
+        if draw_class is None:
+            return False
 
+        obj = draw_class.idraw(self, cxt)
+
+        # update display every delta_time secs
         if obj is not None:
             obj.initialize(self, cxt.viewer, self.logger)
             self._draw_obj = obj
-            if time.time() - self._process_time > self._delta_time:
+            if force_update or (time.time() - self._process_time > self._delta_time):
                 self.process_drawing()
 
         return True
@@ -201,15 +205,17 @@ class DrawingMixin(object):
         crdtype = self.t_drawparams.get('coord', 'data')
         crdmap = viewer.get_coordmap(crdtype)
         x, y = crdmap.data_to(data_x, data_y)
+
+        klass = self.draw_dict.get(self.t_drawtype, None)
+
         # create the drawing context
         self._draw_cxt = Bunch(start_x=x, start_y=y, points=[(x, y)],
                                x=x, y=y, data_x=data_x, data_y=data_y,
                                drawparams=self.t_drawparams,
                                crdmap=crdmap, viewer=viewer,
-                               logger=self.logger)
+                               draw_class=klass, logger=self.logger)
 
-        self._draw_update(data_x, data_y, self._draw_cxt)
-        self.process_drawing()
+        self._draw_update(data_x, data_y, self._draw_cxt, force_update=True)
         return True
 
     def draw_stop(self, canvas, event, data_x, data_y, viewer):
@@ -219,7 +225,7 @@ class DrawingMixin(object):
         self._draw_update(data_x, data_y, self._draw_cxt)
         obj, self._draw_obj = self._draw_obj, None
 
-        if obj:
+        if obj is not None:
             objtag = self.add(obj)
             self.make_callback('draw-event', objtag)
 
@@ -244,12 +250,14 @@ class DrawingMixin(object):
             return False
 
         cxt = self._draw_cxt
-        if self.t_drawtype in ('polygon', 'path'):
+        if self.t_drawtype in ('polygon', 'freepolygon', 'path', 'freepath'):
             x, y = cxt.crdmap.data_to(data_x, data_y)
             cxt.points.append((x, y))
         elif self.t_drawtype == 'beziercurve' and len(cxt.points) < 3:
             x, y = cxt.crdmap.data_to(data_x, data_y)
             cxt.points.append((x, y))
+
+        self._draw_update(data_x, data_y, cxt, force_update=True)
         return True
 
     def draw_poly_delete(self, canvas, event, data_x, data_y, viewer):
@@ -257,9 +265,12 @@ class DrawingMixin(object):
             return False
 
         cxt = self._draw_cxt
-        if self.t_drawtype in ('polygon', 'path', 'beziercurve'):
+        if self.t_drawtype in ('polygon', 'freepolygon', 'path',
+                               'freepath', 'beziercurve'):
             if len(cxt.points) > 0:
                 cxt.points.pop()
+
+        self._draw_update(data_x, data_y, cxt, force_update=True)
         return True
 
     def is_drawing(self):
@@ -272,10 +283,11 @@ class DrawingMixin(object):
         self.t_drawparams['color'] = colorname
 
     def set_drawtype(self, drawtype, **drawparams):
-        drawtype = drawtype.lower()
-        assert drawtype in self.drawtypes, \
-               ValueError("Bad drawing type '%s': must be one of %s" % (
-            drawtype, self.drawtypes))
+        if drawtype is not None:
+            drawtype = drawtype.lower()
+            assert drawtype in self.drawtypes, \
+                   ValueError("Bad drawing type '%s': must be one of %s" % (
+                drawtype, self.drawtypes))
         self.t_drawtype = drawtype
         self.t_drawparams = drawparams.copy()
 
@@ -653,21 +665,32 @@ class DrawingMixin(object):
 
     def _do_pick(self, canvas, event, data_x, data_y, cb_name, viewer):
         # check for objects at this location
-        objs = canvas.select_items_at(viewer, data_x, data_y)#,
-        #test=self._is_editable)
+        objs = canvas.select_items_at(viewer, data_x, data_y)
 
         if len(objs) == 0:
             # <-- no objects under cursor
+
+            if self._pick_cur_obj is not None:
+                # leaving an object that we were in--make pick-leave cb
+                obj, self._pick_cur_obj = self._pick_cur_obj, None
+                pt = obj.crdmap.data_to(data_x, data_y)
+                obj.make_callback('pick-leave', canvas, event, pt)
+
             return False
 
         # pick top object
         obj = objs[-1]
-        self.logger.info("%s event in %s obj at x, y = %d, %d" % (
+        self.logger.debug("%s event in %s obj at x, y = %d, %d" % (
             cb_name, obj.kind, data_x, data_y))
 
         # get coordinates in native form for this object
         pt = obj.crdmap.data_to(data_x, data_y)
 
+        if self._pick_cur_obj is None:
+            # entering a new object--make pick-enter cb
+            self._pick_cur_obj = obj
+            obj.make_callback('pick-enter', canvas, event, pt)
+
         # make pick callback
         obj.make_callback(cb_name, canvas, event, pt)
         return True
@@ -680,6 +703,10 @@ class DrawingMixin(object):
         return self._do_pick(canvas, event, data_x, data_y,
                              'pick-move', viewer)
 
+    def pick_hover(self, canvas, event, data_x, data_y, viewer):
+        return self._do_pick(canvas, event, data_x, data_y,
+                             'pick-hover', viewer)
+
     def pick_stop(self, canvas, event, data_x, data_y, viewer):
         return self._do_pick(canvas, event, data_x, data_y,
                              'pick-up', viewer)
