@@ -150,12 +150,12 @@ class Image(OnePointMixin, CanvasObjectBase):
             ((x0, y0), (x1, y1), (x2, y2), (x3, y3)) = viewer.get_pan_rect()
             xmin = int(min(x0, x1, x2, x3))
             ymin = int(min(y0, y1, y2, y3))
-            xmax = int(max(x0, x1, x2, x3))
-            ymax = int(max(y0, y1, y2, y3))
+            xmax = int(numpy.ceil(max(x0, x1, x2, x3)))
+            ymax = int(numpy.ceil(max(y0, y1, y2, y3)))
 
             # destination location in data_coords
             #dst_x, dst_y = self.x, self.y + ht
-            dst_x, dst_y = self.x, self.y
+            dst_x, dst_y = self.crdmap.to_data(self.x, self.y)
 
             a1, b1, a2, b2 = 0, 0, self.image.width, self.image.height
 
@@ -391,11 +391,11 @@ class NormImage(Image):
             ((x0, y0), (x1, y1), (x2, y2), (x3, y3)) = viewer.get_pan_rect()
             xmin = int(min(x0, x1, x2, x3))
             ymin = int(min(y0, y1, y2, y3))
-            xmax = int(max(x0, x1, x2, x3))
-            ymax = int(max(y0, y1, y2, y3))
+            xmax = int(numpy.ceil(max(x0, x1, x2, x3)))
+            ymax = int(numpy.ceil(max(y0, y1, y2, y3)))
 
             # destination location in data_coords
-            dst_x, dst_y = self.x, self.y
+            dst_x, dst_y = self.crdmap.to_data(self.x, self.y)
 
             a1, b1, a2, b2 = 0, 0, self.image.width, self.image.height
 
