@@ -8,7 +8,7 @@
 import sys, os
 import logging
 
-from ginga.gtk3w.ImageViewCanvasGtk import ImageViewCanvas
+from ginga.gtk3w.ImageViewGtk import CanvasView, ScrolledView
 from ginga.gtk3w import GtkHelp
 from ginga import AstroImage
 
@@ -30,7 +30,8 @@ class FitsViewer(object):
         self.select = GtkHelp.FileSelection(root)
         vbox = Gtk.VBox(spacing=2)
 
-        fi = ImageViewCanvas(logger)
+        # create the ginga viewer and configure it
+        fi = CanvasView(logger)
         fi.enable_autocuts('on')
         fi.set_autocut_params('zscale')
         fi.enable_autozoom('on')
@@ -39,13 +40,17 @@ class FitsViewer(object):
         fi.ui_setActive(True)
         self.fitsimage = fi
 
+        # enable some user interaction
         bd = fi.get_bindings()
         bd.enable_all(True)
 
         w = fi.get_widget()
         w.set_size_request(512, 512)
 
-        vbox.pack_start(w, True, True, 0)
+        # add scrollbar interface around this viewer
+        si = ScrolledView(fi)
+
+        vbox.pack_start(si, True, True, 0)
 
         hbox = Gtk.HButtonBox()
         hbox.set_layout(Gtk.ButtonBoxStyle.END)
