@@ -11,7 +11,6 @@ import logging
 from ginga.qtw.QtHelp import QtGui, QtCore
 
 from ginga import AstroImage, colors
-from ginga.qtw.ImageViewCanvasQt import ImageViewCanvas
 from ginga.qtw.ImageViewQt import CanvasView
 from ginga.canvas.CanvasObject import get_canvas_types
 from ginga.misc import log
@@ -27,7 +26,6 @@ class FitsViewer(QtGui.QMainWindow):
         self.drawcolors = colors.get_colors()
         self.dc = get_canvas_types()
 
-        #fi = ImageViewCanvas(logger, render='widget')
         fi = CanvasView(logger, render='widget')
         fi.enable_autocuts('on')
         fi.set_autocut_params('zscale')
@@ -54,21 +52,18 @@ class FitsViewer(QtGui.QMainWindow):
         # add canvas to view
         #fi.add(canvas)
         private_canvas = fi.get_canvas()
+        private_canvas.register_for_cursor_drawing(fi)
         private_canvas.add(canvas)
         canvas.ui_setActive(True)
         self.drawtypes = canvas.get_drawtypes()
         self.drawtypes.sort()
 
         # add a color bar
-        private_canvas.add(self.dc.ColorBar(side='bottom', offset=10))
+        #fi.show_color_bar(True)
+        fi.show_focus_indicator(True)
 
-        # add little mode indicator that shows modal states in
-        # the corner
-        private_canvas.add(self.dc.ModeIndicator(corner='ur'))
-        # little hack necessary to get correct operation of the mode indicator
-        # in all circumstances
-        bm = fi.get_bindmap()
-        bm.add_callback('mode-set', lambda *args: fi.redraw(whence=3))
+        # add little mode indicator that shows keyboard modal states
+        fi.show_mode_indicator(True, corner='ur')
 
         w = fi.get_widget()
         w.resize(512, 512)
@@ -150,7 +145,7 @@ class FitsViewer(QtGui.QMainWindow):
         self.canvas.set_drawtype(kind, **params)
 
     def clear_canvas(self):
-        self.canvas.deleteAllObjects()
+        self.canvas.delete_all_objects()
 
     def load_file(self, filepath):
         image = AstroImage.AstroImage(logger=self.logger)
@@ -218,6 +213,22 @@ def main(options, args):
 
     logger = log.get_logger("example2", options=options)
 
+    # Check whether user wants to use OpenCv
+    if options.opencv:
+        from ginga import trcalc
+        try:
+            trcalc.use('opencv')
+        except Exception as e:
+            logger.warning("failed to set OpenCv preference: %s" % (str(e)))
+
+    # Check whether user wants to use OpenCL
+    elif options.opencl:
+        from ginga import trcalc
+        try:
+            trcalc.use('opencl')
+        except Exception as e:
+            logger.warning("failed to set OpenCL preference: %s" % (str(e)))
+
     w = FitsViewer(logger)
     w.resize(524, 540)
     w.show()
@@ -240,17 +251,16 @@ if __name__ == "__main__":
 
     optprs.add_option("--debug", dest="debug", default=False, action="store_true",
                       help="Enter the pdb debugger on main()")
-    optprs.add_option("--log", dest="logfile", metavar="FILE",
-                      help="Write logging output to FILE")
-    optprs.add_option("--loglevel", dest="loglevel", metavar="LEVEL",
-                      type='int', default=logging.INFO,
-                      help="Set logging level to LEVEL")
-    optprs.add_option("--stderr", dest="logstderr", default=False,
+    optprs.add_option("--opencv", dest="opencv", default=False,
                       action="store_true",
-                      help="Copy logging also to stderr")
+                      help="Use OpenCv acceleration")
+    optprs.add_option("--opencl", dest="opencl", default=False,
+                      action="store_true",
+                      help="Use OpenCL acceleration")
     optprs.add_option("--profile", dest="profile", action="store_true",
                       default=False,
                       help="Run the profiler on main()")
+    log.addlogopts(optprs)
 
     (options, args) = optprs.parse_args(sys.argv[1:])
 
