@@ -10,11 +10,12 @@ import numpy
 
 from ginga.gtk3w import GtkHelp
 from ginga import Mixins, Bindings, colors
+from ginga.util.paths import icondir
 import ginga.util.six as six
+
 if six.PY2:
     from ginga.cairow.ImageViewCairo import (ImageViewCairo as ImageView,
                                              ImageViewCairoError as ImageViewError)
-
 else:
     # NOTE [1]: this is a workaround for broken pycairo3--
     # it lacks the ImageSurface.create_for_data() function present in
@@ -27,13 +28,9 @@ else:
 
 from gi.repository import Gtk
 from gi.repository import Gdk
-from gi.repository import GObject
 from gi.repository import GdkPixbuf
 import cairo
 
-moduleHome = os.path.split(sys.modules[__name__].__file__)[0]
-icon_dir = os.path.abspath(os.path.join(moduleHome, '..', 'icons'))
-
 
 class ImageViewGtkError(ImageViewError):
     pass
@@ -58,39 +55,37 @@ class ImageViewGtk(ImageView):
         self.imgwin = imgwin
         self.imgwin.show_all()
 
-        # cursors
-        self.cursor = {}
-
         # see reschedule_redraw() method
-        self._defer_task = None
-        self.msgtask = None
-
+        self._defer_task = GtkHelp.Timer(0.0,
+                                         lambda timer: self.delayed_redraw())
+        self.msgtask = GtkHelp.Timer(0.0,
+                                     lambda timer: self.onscreen_message(None))
 
     def get_widget(self):
         return self.imgwin
 
-    def get_image_as_pixbuf(self):
+    def get_plain_image_as_pixbuf(self):
         arr = self.getwin_array(order='RGB')
         pixbuf = GtkHelp.pixbuf_new_from_array(arr,
                                                GdkPixbuf.Colorspace.RGB,
                                                8)
         return pixbuf
 
-    def get_image_as_widget(self):
+    def get_plain_image_as_widget(self):
         """Used for generating thumbnails.  Does not include overlaid
         graphics.
         """
-        pixbuf = self.get_image_as_pixbuf()
+        pixbuf = self.get_plain_image_as_pixbuf()
         image = Gtk.Image()
         image.set_from_pixbuf(pixbuf)
         image.show()
         return image
 
-    def save_image_as_file(self, filepath, format='png', quality=90):
+    def save_plain_image_as_file(self, filepath, format='png', quality=90):
         """Used for generating thumbnails.  Does not include overlaid
         graphics.
         """
-        pixbuf = self.get_image_as_pixbuf()
+        pixbuf = self.get_plain_image_as_pixbuf()
         options, values = [], []
         if format == 'jpeg':
             options.append('quality')
@@ -102,7 +97,7 @@ class ImageViewGtk(ImageView):
         daht = self.surface.get_height()
         rgb_buf = bytes(self.surface.get_data())
         pixbuf = GtkHelp.pixbuf_new_from_data(rgb_buf, GdkPixbuf.Colorspace.RGB,
-                                      False, 8, dawd, daht, dawd*3)
+                                              False, 8, dawd, daht, dawd*3)
 
         return pixbuf
 
@@ -114,19 +109,8 @@ class ImageViewGtk(ImageView):
         pixbuf.save(filepath, format, options)
 
     def reschedule_redraw(self, time_sec):
-        time_ms = int(time_sec * 1000)
-        try:
-            if self._defer_task is not None:
-                GObject.source_remove(self._defer_task)
-                self._defer_task = None
-        except:
-            pass
-        self._defer_task = GObject.timeout_add(time_ms,
-                                               self.delayed_redraw_gtk)
-
-    def delayed_redraw_gtk(self):
-        self._defer_task = None
-        self.delayed_redraw()
+        self._defer_task.cancel()
+        self._defer_task.start(time_sec)
 
     def update_image(self):
         if not self.surface:
@@ -184,25 +168,14 @@ class ImageViewGtk(ImageView):
         self.configure_window(width, height)
         return True
 
-    def size_request(self, widget, requisition):
-        """Callback function to request our desired size.
-        """
-        requisition.width, requisition.height = self.get_desired_size()
-        return True
-
     def set_cursor(self, cursor):
         win = self.imgwin.get_window()
         if win is not None:
             win.set_cursor(cursor)
 
-    def define_cursor(self, ctype, cursor):
-        self.cursor[ctype] = cursor
-
-    def get_cursor(self, ctype):
-        return self.cursor[ctype]
-
-    def switch_cursor(self, ctype):
-        self.set_cursor(self.cursor[ctype])
+    def make_cursor(self, iconpath, x, y):
+        cursor = GtkHelp.make_cursor(self.imgwin, iconpath, x, y)
+        return cursor
 
     def center_cursor(self):
         if self.imgwin is None:
@@ -233,23 +206,11 @@ class ImageViewGtk(ImageView):
         return buf
 
     def onscreen_message(self, text, delay=None, redraw=True):
-        if self.msgtask is not None:
-            try:
-                GObject.source_remove(self.msgtask)
-                self.msgtask = None
-            except:
-                pass
-        self.message = text
-        if redraw:
-            self.redraw(whence=3)
-        if delay:
-            ms = int(delay * 1000.0)
-            self.msgtask = GObject.timeout_add(ms,
-                                               self.clear_onscreen_message_gtk)
+        self.msgtask.cancel()
+        self.set_onscreen_message(text, redraw=redraw)
+        if delay is not None:
+            self.msgtask.start(delay)
 
-    def clear_onscreen_message_gtk(self):
-        self.msgtask = None
-        self.onscreen_message(None)
 
 class ImageViewEvent(ImageViewGtk):
 
@@ -293,12 +254,6 @@ class ImageViewEvent(ImageViewGtk):
                              Gdk.DragAction.COPY)
         imgwin.drag_dest_add_text_targets()
 
-        # last known window mouse position
-        self.last_win_x = 0
-        self.last_win_y = 0
-        # last known data mouse position
-        self.last_data_x = 0
-        self.last_data_y = 0
         # Does widget accept focus when mouse enters window
         self.enter_focus = self.t_.get('enter_focus', True)
 
@@ -369,9 +324,9 @@ class ImageViewEvent(ImageViewGtk):
 
         # Define cursors
         for curname, filename in (('pan', 'openHandCursor.png'),
-                               ('pick', 'thinCrossCursor.png')):
-            path = os.path.join(icon_dir, filename)
-            cur = GtkHelp.make_cursor(self.imgwin, path, 8, 8)
+                                  ('pick', 'thinCrossCursor.png')):
+            path = os.path.join(icondir, filename)
+            cur = self.make_cursor(path, 8, 8)
             self.define_cursor(curname, cur)
 
         for name in ('motion', 'button-press', 'button-release',
@@ -413,6 +368,7 @@ class ImageViewEvent(ImageViewGtk):
         # without this we do not get key release events if the focus
         # changes to another window
         #Gdk.keyboard_grab(widget.get_window(), False, event.time)
+        #widget.grab_add()
 
         keyname = Gdk.keyval_name(event.keyval)
         keyname = self.transkey(keyname)
@@ -421,6 +377,7 @@ class ImageViewEvent(ImageViewGtk):
 
     def key_release_event(self, widget, event):
         #Gdk.keyboard_ungrab(event.time)
+        #widget.grab_remove()
 
         keyname = Gdk.keyval_name(event.keyval)
         keyname = self.transkey(keyname)
@@ -430,31 +387,33 @@ class ImageViewEvent(ImageViewGtk):
     def button_press_event(self, widget, event):
         # event.button, event.x, event.y
         x = event.x; y = event.y
+        self.last_win_x, self.last_win_y = x, y
+
         button = 0
         if event.button != 0:
             button |= 0x1 << (event.button - 1)
         self.logger.debug("button event at %dx%d, button=%x" % (x, y, button))
 
         data_x, data_y = self.get_data_xy(x, y)
+        self.last_data_x, self.last_data_y = data_x, data_y
+
         return self.make_ui_callback('button-press', button, data_x, data_y)
 
     def button_release_event(self, widget, event):
         # event.button, event.x, event.y
         x = event.x; y = event.y
+        self.last_win_x, self.last_win_y = x, y
+
         button = 0
         if event.button != 0:
             button |= 0x1 << (event.button - 1)
         self.logger.debug("button release at %dx%d button=%x" % (x, y, button))
 
         data_x, data_y = self.get_data_xy(x, y)
+        self.last_data_x, self.last_data_y = data_x, data_y
+
         return self.make_ui_callback('button-release', button, data_x, data_y)
 
-    def get_last_win_xy(self):
-        return (self.last_win_x, self.last_win_y)
-
-    def get_last_data_xy(self):
-        return (self.last_data_x, self.last_data_y)
-
     def motion_notify_event(self, widget, event):
         button = 0
         if event.is_hint:
@@ -480,6 +439,7 @@ class ImageViewEvent(ImageViewGtk):
     def scroll_event(self, widget, event):
         # event.button, event.x, event.y
         x = event.x; y = event.y
+        self.last_win_x, self.last_win_y = x, y
 
         degrees, direction = GtkHelp.get_scroll_info(event)
         self.logger.debug("scroll deg=%f direction=%f" % (
@@ -598,4 +558,133 @@ class CanvasView(ImageViewZoom):
         self.objects[0] = self.private_canvas
 
 
+class ScrolledView(Gtk.Table):
+    """A class that can take a viewer as a parameter and add scroll bars
+    that respond to the pan/zoom levels.
+    """
+
+    def __init__(self, viewer, parent=None):
+        self.viewer = viewer
+        super(ScrolledView, self).__init__(rows=2, columns=2)
+
+        self.set_border_width(0)
+        self.set_row_spacings(0)
+        self.set_col_spacings(0)
+
+        self._adjusting = False
+        self._scrolling = False
+        self.pad = 20
+        self.rng_x = 100.0
+        self.rng_y = 100.0
+
+        xoptions = Gtk.AttachOptions.EXPAND | Gtk.AttachOptions.FILL
+        yoptions = Gtk.AttachOptions.EXPAND | Gtk.AttachOptions.FILL
+
+        # reparent the viewer widget
+        self.v_w = viewer.get_widget()
+        self.attach(self.v_w, 0, 1, 0, 1,
+                    xoptions=xoptions, yoptions=yoptions,
+                    xpadding=0, ypadding=0)
+
+        self.hsb = Gtk.HScrollbar()
+        self.hsb.set_round_digits(4)
+        self.hsb.connect('value-changed', self._scroll_contents)
+        self.attach(self.hsb, 0, 1, 1, 2,
+                    xoptions=Gtk.AttachOptions.FILL, yoptions=0,
+                    xpadding=0, ypadding=0)
+        self.vsb = Gtk.VScrollbar()
+        self.vsb.set_round_digits(4)
+        self.vsb.connect('value-changed', self._scroll_contents)
+        self.attach(self.vsb, 1, 2, 0, 1,
+                    xoptions=0, yoptions=Gtk.AttachOptions.FILL,
+                    xpadding=0, ypadding=0)
+
+        self.viewer.add_callback('redraw', self._calc_scrollbars)
+        self.viewer.add_callback('limits-set',
+                                 lambda v, l: self._calc_scrollbars(v))
+
+        self._calc_scrollbars(self.viewer)
+
+    def get_widget(self):
+        return self
+
+    def _calc_scrollbars(self, viewer):
+        """Calculate and set the scrollbar handles from the pan and
+        zoom positions.
+        """
+        if self._scrolling:
+            return
+
+        # flag that suppresses a cyclical callback
+        self._adjusting = True
+        try:
+            bd = self.viewer.get_bindings()
+            res = bd.calc_pan_pct(self.viewer, pad=self.pad)
+            if res is None:
+                return
+
+            page_x, page_y = (float(res.thm_pct_x * 100),
+                              float(res.thm_pct_y * 100))
+            self.rng_x, self.rng_y = 100 - page_x, 100 - page_y
+            val_x, val_y = (float(res.pan_pct_x * self.rng_x),
+                            float((1.0 - res.pan_pct_y) * self.rng_y))
+
+            upper_x, upper_y = 100.0, 100.0
+
+            adj = self.hsb.get_adjustment()
+            adj.configure(val_x, 0.0, upper_x, 1.0, page_x, page_x)
+            self.hsb.set_adjustment(adj)
+            adj = self.vsb.get_adjustment()
+            adj.configure(val_y, 0.0, upper_y, 1.0, page_y, page_y)
+            self.vsb.set_adjustment(adj)
+
+        finally:
+            self._adjusting = False
+        return True
+
+    def _scroll_contents(self, adj):
+        """Called when the scroll bars are adjusted by the user.
+        """
+        if self._adjusting:
+            return True
+
+        self._scrolling = True
+        try:
+            pos_x = self.hsb.get_value()
+            pos_y = self.vsb.get_value()
+
+            pct_x = pos_x / float(self.rng_x)
+            # invert Y pct because of orientation of scrollbar
+            pct_y = 1.0 - (pos_y / float(self.rng_y))
+
+            bd = self.viewer.get_bindings()
+            bd.pan_by_pct(self.viewer, pct_x, pct_y, pad=self.pad)
+
+        finally:
+            self._scrolling = False
+
+        return True
+
+    def scroll_bars(self, horizontal='on', vertical='on'):
+        if horizontal == 'on':
+            pass
+        elif horizontal == 'off':
+            pass
+        elif horizontal == 'auto':
+            pass
+        else:
+            raise ValueError("Bad scroll bar option: '%s'; should be one of ('on', 'off' or 'auto')" % (horizontal))
+
+        if vertical == 'on':
+            pass
+        elif vertical == 'off':
+            pass
+        elif vertical == 'auto':
+            pass
+        else:
+            raise ValueError("Bad scroll bar option: '%s'; should be one of ('on', 'off' or 'auto')" % (vertical))
+
+        self.viewer.logger.warning("scroll_bar(): settings for gtk3 currently ignored!")
+
+
 #END
