@@ -44,15 +44,24 @@ class WidgetBase(Callback.Callbacks):
     def get_size(self):
         window = self.widget.get_window()
         if window is not None:
+            # NOTE: we don't seem to get accurate information unless
+            # the window is actually mapped, thus the test
             rect = self.widget.get_allocation()
             x, y, wd, ht = rect.x, rect.y, rect.width, rect.height
-            return wd, ht
 
-        # window isn't realized yet--return request size
-        wd, ht = self.widget.size_request()
-        wd, ht = max(0, wd), max(0, ht)
+        else:
+            # window maybe isn't realized yet--try other ways
+            # NOTE: get_size_request() does not seem to work the same
+            # as size_request() for gtk2!
+            wd, ht = self.widget.size_request()
+
         return wd, ht
 
+    def get_pos(self):
+        rect = self.widget.get_allocation()
+        x, y, width, height = rect.x, rect.y, rect.width, rect.height
+        return x, y
+
     def get_app(self):
         return _app
 
@@ -71,6 +80,9 @@ class WidgetBase(Callback.Callbacks):
 
     def resize(self, width, height):
         self.widget.set_size_request(width, height)
+        # hackish way to allow the widget to be resized down again later
+        # NOTE: for gtk2 we can't do this because it affects get_size()
+        #gobject.idle_add(self.widget.set_size_request, -1, -1)
 
     def get_font(self, font_family, point_size):
         font = GtkHelp.get_font(font_family, point_size)
@@ -623,7 +635,11 @@ class StatusBar(WidgetBase):
         self.widget = sbar
         self.statustask = None
 
-    def set_message(self, msg_str):
+    def _clear_message(self):
+        self.statustask = None
+        self.widget.remove_all(self.ctx_id)
+
+    def set_message(self, msg_str, duration=10.0):
         try:
             self.widget.remove_all(self.ctx_id)
         except:
@@ -634,8 +650,8 @@ class StatusBar(WidgetBase):
         # remove message in about 10 seconds
         if self.statustask:
             gobject.source_remove(self.statustask)
-        self.statustask = gobject.timeout_add(10000,
-                                  self.widget.remove_all, self.ctx_id)
+        self.statustask = gobject.timeout_add(int(1000 * duration),
+                                              self._clear_message)
 
 
 class TreeView(WidgetBase):
@@ -1197,43 +1213,28 @@ class StackWidget(TabWidget):
         nb.set_show_tabs(False)
         nb.set_show_border(False)
 
-class MDIWidget(TabWidget):
+class MDIWidget(ContainerBase):
 
     def __init__(self, tabpos='top', mode='tabs'):
-        super(MDIWidget, self).__init__(tabpos=tabpos)
+        super(MDIWidget, self).__init__()
 
-        self.mode = 'tabs'
-        self.true_mdi = False
-
-    def get_mode(self):
-        return self.mode
-
-    def set_mode(self, mode):
-        pass
-
-    def tile_panes(self):
-        pass
-
-    def cascade_panes(self):
-        pass
-
-    def use_tabs(self, tf):
-        pass
-
-
-class MDIWidget2(ContainerBase):
-
-    def __init__(self, tabpos='top', mode='tabs'):
-        super(MDIWidget2, self).__init__()
-
-        self.mode = 'tabs'
-        self.true_mdi = False
+        self.mode = 'mdi'
+        self.true_mdi = True
 
+        # TODO: currently scrollbars are not working
         sw = gtk.ScrolledWindow()
         sw.set_border_width(2)
         sw.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
         self.widget = sw
-        self.mdi_w = GtkHelp.MDIWidget()
+        w = GtkHelp.MDIWidget()
+        self.mdi_w = w
+        w._move_page = w.move_page
+        w.move_page = self._window_moved
+        w._resize_page = w.resize_page
+        w.resize_page = self._window_resized
+
+        sw.set_hadjustment(self.mdi_w.get_hadjustment())
+        sw.set_vadjustment(self.mdi_w.get_vadjustment())
         sw.add(self.mdi_w)
 
     def get_mode(self):
@@ -1246,15 +1247,50 @@ class MDIWidget2(ContainerBase):
         self.add_ref(child)
         child_w = child.get_widget()
         label = gtk.Label(title)
-        self.mdi_w.append_page(child_w, label)
+        subwin = self.mdi_w.append_page(child_w, label)
         #self.mdi_w.show_all()
         # attach title to child
         child.extdata.tab_title = title
 
+        # does child have a previously saved size
+        size = child.extdata.get('mdi_size', None)
+        if size is not None:
+            wd, ht = size
+            self.mdi_w.resize_page(subwin, wd, ht)
+
+        # does child have a previously saved position
+        pos = child.extdata.get('mdi_pos', None)
+        if pos is not None:
+            x, y = pos
+            self.mdi_w.move_page(subwin, x, y)
+
+    def _remove(self, childw, delete=False):
+        self.mdi_w.remove(childw)
+        if delete:
+            childw.destroy()
+
     def _cb_redirect(self, nbw, gptr, index):
         child = self.index_to_widget(index)
         self.make_callback('page-switch', child)
 
+    def _window_resized(self, subwin, wd, ht):
+        self.mdi_w._resize_page(subwin, wd, ht)
+
+        # save size
+        nchild = subwin.widget
+        child = self._native_to_child(nchild)
+        child.extdata.mdi_size = (wd, ht)
+        return True
+
+    def _window_moved(self, subwin, x, y):
+        self.mdi_w._move_page(subwin, x, y)
+
+        # save position
+        nchild = subwin.widget
+        child = self._native_to_child(nchild)
+        child.extdata.mdi_pos = (x, y)
+        return True
+
     def get_index(self):
         return self.mdi_w.get_current_page()
 
@@ -1434,14 +1470,17 @@ class Toolbar(ContainerBase):
             w.set_orientation(gtk.ORIENTATION_VERTICAL)
         self.widget = w
 
-    def add_action(self, text, toggle=False, iconpath=None):
+    def add_action(self, text, toggle=False, iconpath=None, iconsize=None):
         if toggle:
             child = ToggleButton(text)
         else:
             child = Button(text)
 
         if iconpath is not None:
-            pixbuf = GtkHelp.pixbuf_new_from_file_at_size(iconpath, 24, 24)
+            wd, ht = 24, 24
+            if iconsize is not None:
+                wd, ht = iconsize
+            pixbuf = GtkHelp.pixbuf_new_from_file_at_size(iconpath, wd, ht)
             if pixbuf is not None:
                 image = gtk.image_new_from_pixbuf(pixbuf)
                 child.get_widget().set_image(image)
@@ -1454,6 +1493,13 @@ class Toolbar(ContainerBase):
         w = child.get_widget()
         self.widget.append_widget(w, None, None)
 
+    def add_menu(self, text, menu=None):
+        if menu is None:
+            menu = Menu()
+        child = self.add_action(text)
+        child.add_callback('activated', lambda w: menu.popup())
+        return menu
+
     def add_separator(self):
         self.widget.append_space()
 
@@ -1560,6 +1606,7 @@ class TopLevelMixin(object):
         self.widget.connect("destroy", self._quit)
         self.widget.connect("delete_event", self._close_event)
         self.widget.connect("window_state_event", self._window_event)
+        self.widget.connect("configure-event", self._configure_event)
 
         if not title is None:
             self.widget.set_title(title)
@@ -1576,9 +1623,12 @@ class TopLevelMixin(object):
         self.close()
 
     def _close_event(self, widget, event):
-        self.close()
-        # don't automatically destroy window
-        return True
+        try:
+            self.close()
+
+        finally:
+            # don't automatically destroy window
+            return True
 
     def _window_event(self, widget, event):
         if ((event.changed_mask & gtk.gdk.WINDOW_STATE_FULLSCREEN) or
@@ -1587,6 +1637,12 @@ class TopLevelMixin(object):
         else:
             self._fullscreen = False
 
+    def _configure_event(self, widget, event):
+        x, y, width, height = event.x, event.y, event.width, event.height
+        x, y = self.widget.translate_coordinates(self.widget, x, y)
+        self.extdata.setvals(x=x, y=y, width=width, height=height)
+        return False
+
     def close(self):
         ## try:
         ##     self.widget.destroy()
@@ -1596,24 +1652,38 @@ class TopLevelMixin(object):
 
         self.make_callback('close')
 
+    def get_size(self):
+        window = self.widget.get_window()
+        if window is not None:
+            rect = self.widget.get_allocation()
+            x, y, wd, ht = rect.x, rect.y, rect.width, rect.height
+
+        else:
+            # window maybe isn't realized yet--try other ways
+            wd, ht = self.widget.size_request()
+            ed = self.extdata
+            wd, ht = ed.get('width', wd), ed.get('height', ht)
+
+        return wd, ht
+
+    def get_pos(self):
+        window = self.widget.get_window()
+        if window is not None:
+            x, y = window.get_origin()
+        else:
+            ed = self.extdata
+            x, y = ed.get('x', None), ed.get('y', None)
+
+        return x, y
+
     def raise_(self):
         window = self.widget.get_window()
-        ## if window:
-        ##     if hasattr(window, 'present'):
-        ##         # gtk3 ?
-        ##         window.present()
-        ##     else:
-        ##         # gtk2
-        ##         window.show()
         window.raise_()
 
     def lower(self):
         window = self.widget.get_window()
         window.lower()
 
-    def resize(self, width, height):
-        self.widget.set_size_request(width, height)
-
     def focus(self):
         window = self.widget.get_window()
         window.focus()
@@ -1738,8 +1808,7 @@ class Application(Callback.Callbacks):
         return w
 
     def mainloop(self):
-        while True:
-            self.process_events()
+        gtk.main()
 
 
 class Dialog(TopLevelMixin, WidgetBase):
