@@ -128,12 +128,11 @@ class ColorBar(Callback.Callbacks):
         dx = x - self._start_x
         wd, ht = event.viewer.get_window_size()
         pct = float(dx) / float(wd)
-        #print "dx=%f wd=%d pct=%f" % (dx, wd, pct)
         self.shift_colormap(pct)
         return True
 
     def draw_release_cb(self, canvas, event, data_x, data_y):
-        self.rgbmap.reset_cmap()
+        self.rgbmap.restore_cmap()
         return True
 
     def cursor_drag_cb(self, canvas, event, data_x, data_y):
@@ -142,7 +141,6 @@ class ColorBar(Callback.Callbacks):
 
         dx = x - self._start_x
         pct = float(dx) / float(wd)
-        #print "dx=%f wd=%d pct=%f" % (dx, wd, pct)
         self.shift_colormap(pct)
         return True
 
