@@ -1,9 +1,6 @@
 #
 # Desktop.py -- Generic widgets Desktop GUI layout
 #
-# Eric Jeschke (eric@naoj.org)
-#
-# Copyright (c) Eric R. Jeschke.  All rights reserved.
 # This is open-source software licensed under a BSD license.
 # Please see the file LICENSE.txt for details.
 #
@@ -26,6 +23,8 @@ class Desktop(Callback.Callbacks):
         self.workspace = Bunch.caselessDict()
 
         self.toplevels = []
+        self.node = {}
+        self.node_idx = 0
         self._cur_dialogs = []
 
         for name in ('page-switch', 'all-closed'):
@@ -35,13 +34,14 @@ class Desktop(Callback.Callbacks):
 
     def make_ws(self, name, group=1, show_tabs=True, show_border=False,
                 detachable=False, tabpos=None, scrollable=True, closeable=False,
-                wstype='tabs'):
+                wstype='tabs', use_toolbar=False):
 
         if (wstype in ('nb', 'ws', 'tabs')) and (not show_tabs):
             wstype = 'stack'
 
         ws = Workspace(name=name, wstype=wstype,
-                       group=group, detachable=detachable)
+                       group=group, detachable=detachable,
+                       use_toolbar=use_toolbar)
         ws.add_callback('page-switch', self.switch_page_cb)
         ws.add_callback('page-detach', self.page_detach_cb)
 
@@ -53,22 +53,24 @@ class Desktop(Callback.Callbacks):
         if (wstype == 'tabs') and show_tabs:
             nb.set_tab_position(tabpos)
 
-        ## vbox = Widgets.VBox()
-        ## toolbar = Widgets.Toolbar(orientation='horizontal')
-        ## vbox.add_widget(toolbar, stretch=0)
-        ## vbox.add_widget(nb, stretch=1)
-
-        ## # create a Workspace pulldown menu, and add it to the menu bar
-        ## winbtn = toolbar.add_action(text="Workspace")
-
-        ## winmenu = Widgets.Menu()
-        ## item = winmenu.add_name("Take Tab")
-        ## item.add_callback('activated',
-        ##                   lambda *args: self.take_tab_cb(nb, args))
+        # # create a Workspace pulldown menu, and add it to the tool bar
+        # if ws.toolbar is not None:
+        #     winmenu = ws.toolbar.add_menu("Workspace")
+        #     item = winmenu.add_name("Take Tab")
+        #     item.add_callback('activated',
+        #                       lambda *args: self.take_tab_cb(nb, args))
 
         self.workspace[name] = ws
         return ws
 
+    def delete_ws(self, name):
+        if not self.has_ws(name):
+            raise ValueError("No workspace named '%s'" % (name))
+
+        # NOTE: currently the Desktop object does not close the window
+        # or tab--you must manage that yourself
+        del self.workspace[name]
+
     def has_ws(self, name):
         return name in self.workspace
 
@@ -115,10 +117,11 @@ class Desktop(Callback.Callbacks):
                 tabname = 'tab%d' % self.tabcount
 
         ws.add_tab(widget, title=labelname)
-        self.tab[tabname] = Bunch.Bunch(widget=widget, name=labelname,
-                                        tabname=tabname, data=data,
-                                        group=group, wsname=wsname)
-        return tabname
+        bnch = Bunch.Bunch(widget=widget, name=labelname,
+                           tabname=tabname, data=data,
+                           group=group, wsname=wsname)
+        self.tab[tabname] = bnch
+        return bnch
 
     def _find_nb(self, tabname):
         widget = self.tab[tabname].widget
@@ -153,7 +156,7 @@ class Desktop(Callback.Callbacks):
             nb, index = self._find_nb(name)
             if (nb is not None) and (index >= 0):
                 nb.set_index(index)
-            
+
     def remove_tab(self, tabname):
         nb, index = self._find_nb(tabname)
         widget = self.tab[tabname].widget
@@ -166,24 +169,6 @@ class Desktop(Callback.Callbacks):
         if (nb is not None) and hasattr(nb, 'highlight_tab'):
             nb.highlight_tab(index, onoff)
 
-    def _add_toolbar(self, vbox, ws):
-        toolbar = Widgets.Toolbar()
-        vbox.add_widget(toolbar, stretch=0)
-
-        # create a Workspace pulldown menu, and add it to the menu bar
-        winmenu = toolbar.add_name("Workspace")
-
-        item = winmenu.add_name("Take Tab")
-        item.add_callback('activated',
-                          lambda *args: self.take_tab_cb(ws.widget, args))
-
-        ## winmenu.add_separator()
-
-        ## closeitem = winmenu.add_name("Close")
-        ## #bnch.widget.closeEvent = lambda event: self.close_page_cb(bnch, event)
-        ## closeitem.add_callback('activated',
-        ##                        lambda *args: self._close_page(ws))
-
     def show_dialog(self, dialog):
         dialog.show()
         # save a handle so widgets aren't garbage collected
@@ -209,12 +194,17 @@ class Desktop(Callback.Callbacks):
         vbox = Widgets.VBox()
         vbox.set_border_width(0)
         topw.set_widget(vbox)
-        self._add_toolbar(vbox)
 
         vbox.add_widget(bnch.widget, stretch=1)
         topw.show()
         return topw
 
+    def remove_toplevel(self, widget):
+        widget.hide()
+        if widget in self.toplevels:
+            self.toplevels.remove(widget)
+        widget.delete()
+
     def create_toplevel_ws(self, wsname, width, height,
                            group=2, x=None, y=None):
         # create main frame
@@ -222,13 +212,11 @@ class Desktop(Callback.Callbacks):
 
         # TODO: this needs to be more sophisticated
         ## root.set_title(wsname)
-        ws = self.make_ws(wsname, wstype='tabs')
+        ws = self.make_ws(wsname, wstype='tabs', use_toolbar=True)
 
         vbox = Widgets.VBox()
         vbox.set_border_width(0)
 
-        self._add_toolbar(vbox, ws)
-
         vbox.add_widget(bnch.widget)
         root.set_widget(vbox)
 
@@ -283,18 +271,29 @@ class Desktop(Callback.Callbacks):
             self.make_callback('page-switch', bnch.name, bnch.data)
         return False
 
-    def make_desktop(self, layout, widgetDict=None):
-        if widgetDict is None:
-            widgetDict = {}
+    def make_desktop(self, layout, widget_dict=None):
+        if widget_dict is None:
+            widget_dict = {}
+        self.node = {}
+        self.node_idx = 0
 
-        def process_common_params(widget, inparams):
-            params = Bunch.Bunch(name=None, height=-1, width=-1,
-                                 xpos=-1, ypos=-1, spacing=None,
-                                 wexp=None, hexp=None)
-            params.update(inparams)
+        def process_common_params(kind, widget, params):
+            subst_params = Bunch.Bunch(name=None, height=-1, width=-1,
+                                       xpos=-1, ypos=-1, spacing=None,
+                                       wexp=None, hexp=None)
+            subst_params.update(params)
+            params.update(subst_params)
 
-            if params.name:
-                widgetDict[params.name] = widget
+            if params.name is not None:
+                name = params.name
+                widget_dict[params.name] = widget
+            else:
+                name = '%s_%d' % (kind, self.node_idx)
+                self.node_idx += 1
+
+            bnch = Bunch.Bunch(kind=kind, widget=widget,
+                               name=name, params=params)
+            self.node[name] = bnch
 
             wexp, hexp = params.wexp, params.hexp
 
@@ -336,16 +335,19 @@ class Desktop(Callback.Callbacks):
                 if params.xpos >= 0:
                     widget.move(params.xpos, params.ypos)
 
-        def make_widget(kind, paramdict, args, pack):
+            return bnch
+
+        def make_widget(kind, params, args, pack):
             kind = kind.lower()
 
             # Process workspace parameters
-            params = Bunch.Bunch(name=None, title=None, height=-1,
-                                 width=-1, group=1, show_tabs=True,
-                                 show_border=False, scrollable=True,
-                                 detachable=False, wstype='tabs',
-                                 tabpos='top')
-            params.update(paramdict)
+            param_dict = Bunch.Bunch(name=None, title=None, height=-1,
+                                      width=-1, group=1, show_tabs=True,
+                                      show_border=False, scrollable=True,
+                                      detachable=False, wstype='tabs',
+                                      tabpos='top', use_toolbar=False)
+            param_dict.update(params)
+            params.update(param_dict)
 
             if kind == 'widget':
                 widget = args[0]
@@ -353,12 +355,13 @@ class Desktop(Callback.Callbacks):
             elif kind == 'ws':
                 group = int(params.group)
                 ws = self.make_ws(params.name, group=group,
-                                      show_tabs=params.show_tabs,
-                                      show_border=params.show_border,
-                                      detachable=params.detachable,
-                                      tabpos=params.tabpos,
-                                      wstype=params.wstype,
-                                      scrollable=params.scrollable)
+                                  show_tabs=params.show_tabs,
+                                  show_border=params.show_border,
+                                  detachable=params.detachable,
+                                  tabpos=params.tabpos,
+                                  wstype=params.wstype,
+                                  scrollable=params.scrollable,
+                                  use_toolbar=params.use_toolbar)
                 widget = ws.widget
                 #debug(widget)
 
@@ -371,8 +374,9 @@ class Desktop(Callback.Callbacks):
             else:
                 pack(widget)
 
-            #process_common_params(widget, params)
+            #process_common_params(kind, widget, params)
 
+            res = []
             if (kind in ('ws', 'mdi', 'grid', 'stack')) and (len(args) > 0):
                 # <-- Workspace specified a sub-layout.  We expect a list
                 # of tabname, layout pairs--iterate over these and add them
@@ -383,67 +387,82 @@ class Desktop(Callback.Callbacks):
                         self.add_tab(params.name, w, group,
                                      tabname, tabname.lower())
 
-                    make(layout, pack)
+                    res.append((tabname, make(layout, pack)))
 
-            process_common_params(widget, params)
+            bnch = process_common_params(kind, widget, params)
+            if kind == 'ws':
+                bnch.ws = ws
 
-            #return widget
+            #return [kind, params] + res
+            return [kind, params, res]
 
         # Horizontal adjustable panel
         def hpanel(params, cols, pack):
+            res = []
             if len(cols) >= 2:
                 widget = Widgets.Splitter(orientation='horizontal')
-                process_common_params(widget, params)
+                process_common_params('hpanel', widget, params)
 
                 sizes = []
                 for col in cols:
-                    make(col, lambda w: widget.add_widget(w))
+                    c = make(col, lambda w: widget.add_widget(w))
+                    res.append(c)
 
                     # collect widths to set width of panes
                     params = col[1]
                     if 'width' in params:
                         sizes.append(params['width'])
 
+                sizes = params.get('sizes', sizes)
                 if len(sizes) == len(cols):
                     widget.set_sizes(sizes)
+                    params.update(dict(sizes=sizes))
 
             elif len(cols) == 1:
                 widget = Widgets.HBox()
                 widget.set_border_width(0)
-                process_common_params(widget, params)
+                process_common_params('hbox', widget, params)
 
-                make(cols[0], lambda w: widget.add_widget(w, stretch=1))
+                c = make(cols[0], lambda w: widget.add_widget(w, stretch=1))
+                res.append(c)
                 #widget.show()
 
             pack(widget)
+            return ['hpanel', params] + res
 
         # Vertical adjustable panel
         def vpanel(params, rows, pack):
+            res = []
             if len(rows) >= 2:
                 widget = Widgets.Splitter(orientation='vertical')
-                process_common_params(widget, params)
+                process_common_params('vpanel', widget, params)
 
                 sizes = []
                 for row in rows:
-                    make(row, lambda w: widget.add_widget(w))
+                    r = make(row, lambda w: widget.add_widget(w))
+                    res.append(r)
 
                     # collect heights to set height of panes
                     params = row[1]
                     if 'height' in params:
                         sizes.append(params['height'])
 
+                sizes = params.get('sizes', sizes)
                 if len(sizes) == len(rows):
                     widget.set_sizes(sizes)
+                    params.update(dict(sizes=sizes))
 
             elif len(rows) == 1:
                 widget = Widgets.VBox()
                 widget.set_border_width(0)
-                process_common_params(widget, params)
+                process_common_params('vbox', widget, params)
 
-                make(rows[0], lambda w: widget.add_widget(w, stretch=1))
+                r = make(rows[0], lambda w: widget.add_widget(w, stretch=1))
+                res.append(r)
                 #widget.show()
 
             pack(widget)
+            return ['vpanel', params] + res
 
         # Horizontal fixed array
         def hbox(params, cols, pack):
@@ -451,6 +470,7 @@ class Desktop(Callback.Callbacks):
             widget.set_border_width(0)
             widget.set_spacing(0)
 
+            res = []
             for dct in cols:
                 if isinstance(dct, dict):
                     stretch = dct.get('stretch', 0)
@@ -460,10 +480,14 @@ class Desktop(Callback.Callbacks):
                     stretch = 0
                     col = dct
                 if col is not None:
-                    make(col, lambda w: widget.add_widget(w,
-                                                          stretch=stretch))
-            process_common_params(widget, params)
+                    c = make(col, lambda w: widget.add_widget(w,
+                                                              stretch=stretch))
+                    c = dict(col=c, stretch=stretch)
+                    res.append(c)
+
+            process_common_params('hbox', widget, params)
             pack(widget)
+            return ['hbox', params] + res
 
         # Vertical fixed array
         def vbox(params, rows, pack):
@@ -471,6 +495,7 @@ class Desktop(Callback.Callbacks):
             widget.set_border_width(0)
             widget.set_spacing(0)
 
+            res = []
             for dct in rows:
                 if isinstance(dct, dict):
                     stretch = dct.get('stretch', 0)
@@ -480,10 +505,14 @@ class Desktop(Callback.Callbacks):
                     stretch = 0
                     row = dct
                 if row is not None:
-                    make(row, lambda w: widget.add_widget(w,
-                                                          stretch=stretch))
-            process_common_params(widget, params)
+                    r = make(row, lambda w: widget.add_widget(w,
+                                                              stretch=stretch))
+                    r = dict(row=r, stretch=stretch)
+                    res.append(r)
+
+            process_common_params('vbox', widget, params)
             pack(widget)
+            return ['vbox', params] + res
 
         # Sequence of separate top-level items
         def seq(params, cols, pack):
@@ -497,7 +526,9 @@ class Desktop(Callback.Callbacks):
                 w_top.set_widget(w)
                 self.toplevels.append(w_top)
                 w_top.show()
+                process_common_params('top', w_top, params)
 
+            res = []
             for dct in cols:
                 if isinstance(dct, dict):
                     stretch = dct.get('stretch', 0)
@@ -507,56 +538,163 @@ class Desktop(Callback.Callbacks):
                     stretch = 0
                     col = dct
                 if col is not None:
-                    make(col, mypack)
+                    res.append(make(col, mypack))
 
             widget = Widgets.Label("Placeholder")
             pack(widget)
+            return ['seq', params] + res
 
         def make(constituents, pack):
             kind = constituents[0]
-            params = constituents[1]
+            params = Bunch.Bunch(constituents[1])
             if len(constituents) > 2:
                 rest = constituents[2:]
             else:
                 rest = []
 
             if kind == 'vpanel':
-                vpanel(params, rest, pack)
+                res = vpanel(params, rest, pack)
             elif kind == 'hpanel':
-                hpanel(params, rest, pack)
+                res = hpanel(params, rest, pack)
             elif kind == 'vbox':
-                vbox(params, rest, pack)
+                res = vbox(params, rest, pack)
             elif kind == 'hbox':
-                hbox(params, rest, pack)
+                res = hbox(params, rest, pack)
             elif kind == 'seq':
-                seq(params, rest, pack)
+                res = seq(params, rest, pack)
             elif kind in ('ws', 'mdi', 'widget'):
-                make_widget(kind, params, rest, pack)
+                res = make_widget(kind, params, rest, pack)
 
-        make(layout, lambda w: None)
+            return res
 
+        self.layout = make(layout, lambda w: None)
+        return self.layout
+
+    def record_sizes(self):
+        """Record sizes of all container widgets in the layout.
+        The sizes are recorded in the `params` mappings in the layout.
+        """
+        for rec in self.node.values():
+            w = rec.widget
+            wd, ht = w.get_size()
+
+            rec.params.update(dict(name=rec.name, width=wd, height=ht))
+
+            if rec.kind in ('hpanel', 'vpanel'):
+                sizes = w.get_sizes()
+                rec.params.update(dict(sizes=sizes))
+
+            if rec.kind in ('ws',):
+                # record ws type
+                # TODO: record positions of subwindows
+                rec.params.update(dict(wstype=rec.ws.wstype))
+
+            if rec.kind == 'top':
+                # record position for top-level widgets as well
+                x, y = w.get_pos()
+                if x is not None and y is not None:
+                    # we don't always get reliable information about position
+                    rec.params.update(dict(xpos=x, ypos=y))
+
+    def write_layout_conf(self, lo_file, layout=None):
+        if layout is None:
+            layout = self.layout
+
+        import pprint
+        self.record_sizes()
+
+        # write layout
+        with open(lo_file, 'w') as out_f:
+            pprint.pprint(layout, out_f)
+
+    def read_layout_conf(self, lo_file):
+        import ast
+
+        # read layout
+        with open(lo_file, 'r') as in_f:
+            buf = in_f.read()
+
+        layout = ast.literal_eval(buf)
+        return layout
+
+    def build_desktop(self, layout, lo_file=None, widget_dict=None):
+        if lo_file is not None:
+            alt_layout = layout
+            try:
+                layout = self.read_layout_conf(lo_file)
+
+            except Exception as e:
+                self.logger.error("Error reading saved layout: %s" % (str(e)))
+                layout = alt_layout
+
+        return self.make_desktop(layout, widget_dict=widget_dict)
 
     ##### WORKSPACES #####
 
 class Workspace(Widgets.WidgetBase):
 
-    def __init__(self, name, wstype='tab', group=0, detachable=False):
+    def __init__(self, name, wstype='tab', group=0, detachable=False,
+                 use_toolbar=False):
         super(Workspace, self).__init__()
 
         self.name = name
         self.wstype = wstype
         self.wstypes = ['tabs', 'mdi', 'stack', 'grid']
         self.vbox = Widgets.VBox()
+        self.vbox.set_spacing(0)
+        self.vbox.set_border_width(0)
         # for now
         self.widget = self.vbox
         self.nb = None
         self.group = group
         self.detachable = detachable
+        self.toolbar = None
+        self.mdi_menu = None
+        self.wstypes_c = ['Tabs', 'Grid', 'MDI', 'Stack']
+        self.wstypes_l = ['tabs', 'grid', 'mdi', 'stack']
+        self.extdata = Bunch.Bunch()
+
+        if use_toolbar:
+            self.toolbar = Widgets.Toolbar(orientation='horizontal')
+            self.toolbar.set_border_width(0)
+            self.vbox.add_widget(self.toolbar, stretch=0)
+
+            ws_menu = self.toolbar.add_menu("Workspace")
+            item = ws_menu.add_name("Close")
+            item.add_callback('activated', self._close_menuitem_cb)
+
 
         self._set_wstype(wstype)
         self.vbox.add_widget(self.nb, stretch=1)
 
-        for name in ('page-switch', 'page-detach'):
+        if use_toolbar:
+            cbox = Widgets.ComboBox()
+            for _wstype in self.wstypes_c:
+                cbox.append_text(_wstype)
+            idx = self.wstypes_l.index(wstype)
+            cbox.set_index(idx)
+            cbox.add_callback('activated', self.config_wstype_cb)
+            cbox.set_tooltip("Set workspace type")
+            self.toolbar.add_widget(cbox)
+
+            mdi_menu = self.toolbar.add_menu("MDI")
+            ## item = mdi_menu.add_name("Panes as Tabs", checkable=True)
+            ## item.add_callback('activated',
+            ##                   lambda w, tf: self.tabstoggle_cb(tf))
+            ## is_tabs = (self.nb.get_mode() == 'tabs')
+            ## item.set_state(is_tabs)
+
+            item = mdi_menu.add_name("Tile Panes")
+            item.add_callback('activated',
+                              lambda *args: self.tile_panes_cb())
+
+            item = mdi_menu.add_name("Cascade Panes")
+            item.add_callback('activated',
+                              lambda *args: self.cascade_panes_cb())
+            self.mdi_menu = mdi_menu
+            self._update_mdi_menu()
+
+        for name in ('page-switch', 'page-detach', 'ws-close'):
             self.enable_callback(name)
 
     def _set_wstype(self, wstype):
@@ -564,6 +702,7 @@ class Workspace(Widgets.WidgetBase):
             wstype = 'tabs'
             self.nb = Widgets.TabWidget(detachable=self.detachable,
                                             group=self.group)
+
         elif wstype == 'mdi':
             self.nb = Widgets.MDIWidget(mode='mdi')
 
@@ -573,6 +712,7 @@ class Workspace(Widgets.WidgetBase):
         elif wstype == 'grid':
             self.nb = SymmetricGridWidget()
 
+        self._update_mdi_menu()
         if self.nb.has_callback('page-switch'):
             self.nb.add_callback('page-switch', self._switch_page_cb)
         if self.nb.has_callback('page-detach'):
@@ -580,6 +720,26 @@ class Workspace(Widgets.WidgetBase):
 
         self.wstype = wstype
 
+    def _update_mdi_menu(self):
+        if self.mdi_menu is None:
+            return
+        if isinstance(self.nb, Widgets.MDIWidget) and self.nb.true_mdi:
+            self.mdi_menu.set_enabled(True)
+        else:
+            self.mdi_menu.set_enabled(False)
+
+    def tile_panes_cb(self):
+        if isinstance(self.nb, Widgets.MDIWidget) and self.nb.true_mdi:
+            self.nb.tile_panes()
+
+    def cascade_panes_cb(self):
+        if isinstance(self.nb, Widgets.MDIWidget) and self.nb.true_mdi:
+            self.nb.cascade_panes()
+
+    def tabstoggle_cb(self, tf):
+        if isinstance(self.nb, Widgets.MDIWidget) and self.nb.true_mdi:
+            self.nb.use_tabs(tf)
+
     def _switch_page_cb(self, nb, child):
         self.focus_index()
         self.make_callback('page-switch', child)
@@ -587,6 +747,9 @@ class Workspace(Widgets.WidgetBase):
     def _detach_page_cb(self, nb, child):
         self.make_callback('page-detach', child)
 
+    def _close_menuitem_cb(self, *args):
+        self.make_callback('ws-close')
+
     def configure_wstype(self, wstype):
         old_widget = self.nb
         self.vbox.remove(old_widget)
@@ -610,6 +773,10 @@ class Workspace(Widgets.WidgetBase):
             self.nb.set_index(idx)
         self.focus_index()
 
+    def config_wstype_cb(self, w, idx):
+        wstype = self.wstypes_l[idx]
+        self.configure_wstype(wstype)
+
     def cycle_wstype(self):
         idx = self.wstypes.index(self.wstype)
         idx = (idx + 1) % len(self.wstypes)
