@@ -14,9 +14,7 @@ from io import BytesIO
 from ginga import ImageView, Mixins, Bindings
 from ginga.util.io_rgb import RGBFileHandler
 from ginga.mockw.CanvasRenderMock import CanvasRenderer
-
-moduleHome = os.path.split(sys.modules[__name__].__file__)[0]
-icon_dir = os.path.abspath(os.path.join(moduleHome, '..', 'icons'))
+from ginga.util.paths import icondir
 
 
 class ImageViewMockError(ImageView.ImageViewError):
@@ -39,7 +37,6 @@ class ImageViewMock(ImageView.ImageViewBase):
         self.t_.setDefaults(show_pan_position=False,
                             onscreen_ff='Sans Serif')
 
-        self.message = None
         # This holds the off-screen pixel map.  It's creation is usually
         # deferred until we know the final size of the window.
         self.pixmap = None
@@ -47,9 +44,6 @@ class ImageViewMock(ImageView.ImageViewBase):
         # create it here
         self.imgwin = None
 
-        # cursors
-        self.cursor = {}
-
         # override default
         #self.defer_redraw = False
         self.rgb_fh = RGBFileHandler(self.logger)
@@ -85,18 +79,6 @@ class ImageViewMock(ImageView.ImageViewBase):
         ##                   data,
         ##                   Rect(0, 0, width, height))
 
-        # Draw a cross in the center of the window in debug mode
-        if self.t_['show_pan_position']:
-            ctr_x, ctr_y = self.get_center()
-            #painter.drawLine(ctr_x - 10, ctr_y, ctr_x + 10, ctr_y)
-            #painter.drawLine(ctr_x, ctr_y - 10, ctr_x, ctr_y + 10)
-
-        # render self.message
-        if self.message:
-            y = ((imgwin_ht // 3) * 2) - (ht // 2)
-            x = (imgwin_wd // 2) - (wd // 2)
-            #painter.drawText(x, y, message)
-
 
     def render_image(self, rgbobj, dst_x, dst_y):
         """Render the image represented by (rgbobj) at dst_x, dst_y
@@ -131,6 +113,9 @@ class ImageViewMock(ImageView.ImageViewBase):
 
         self.configure(width, height)
 
+    def get_image_as_array(self):
+        return self.getwin_array(order=self.get_rgb_order())
+
     def get_rgb_image_as_buffer(self, output=None, format='png',
                                 quality=90):
         # copy pixmap to buffer
@@ -140,16 +125,6 @@ class ImageViewMock(ImageView.ImageViewBase):
                                          output=output)
         return fmt_buf
 
-    def get_image_as_buffer(self, output=None):
-        return self.get_rgb_image_as_buffer(self, output=output)
-
-    def get_image_as_array(self):
-        return self.getwin_array(order=self.get_rgb_order())
-
-    def get_rgb_image_as_bytes(self, format='png', quality=90):
-        buf = self.get_rgb_image_as_buffer(format=format, quality=quality)
-        return buf
-
     def get_rgb_image_as_widget(self, output=None, format='png',
                                 quality=90):
         imgwin_wd, imgwin_ht = self.get_window_size()
@@ -166,7 +141,7 @@ class ImageViewMock(ImageView.ImageViewBase):
         # a file
         res = img_w.save(filepath, format=format, quality=quality)
 
-    def get_image_as_widget(self):
+    def get_plain_image_as_widget(self):
         """Used for generating thumbnails.  Does not include overlaid
         graphics.
         """
@@ -176,11 +151,11 @@ class ImageViewMock(ImageView.ImageViewBase):
         image_w = self._get_wimage(arr)
         return image_w
 
-    def save_image_as_file(self, filepath, format='png', quality=90):
+    def save_plain_image_as_file(self, filepath, format='png', quality=90):
         """Used for generating thumbnails.  Does not include overlaid
         graphics.
         """
-        img_w = self.get_image_as_widget()
+        img_w = self.get_plain_image_as_widget()
         # assumes that the image widget has some method for saving to
         # a file
         res = qimg.save(filepath, format=format, quality=quality)
@@ -207,14 +182,12 @@ class ImageViewMock(ImageView.ImageViewBase):
             # set the cursor on self.imgwin
             pass
 
-    def define_cursor(self, ctype, cursor):
-        self.cursor[ctype] = cursor
-
-    def get_cursor(self, ctype):
-        return self.cursor[ctype]
-
-    def switch_cursor(self, ctype):
-        self.set_cursor(self.cursor[ctype])
+    def make_cursor(self, iconpath, x, y):
+        # return a cursor in the widget set's instance type
+        # iconpath usually refers to a PNG file and x/y is the
+        # cursor hot spot
+        cursorw = None
+        return cursorw
 
     def _get_wimage(self, arr_np):
         """Convert the numpy array (which is in our expected order)
@@ -229,25 +202,16 @@ class ImageViewMock(ImageView.ImageViewBase):
         clr = (r, g, b)
         return clr
 
-    def onscreen_message(self, text, delay=None):
+    def onscreen_message(self, text, delay=None, redraw=True):
         # stop any scheduled updates of the message
 
         # set new message text
-        self.message = text
-        self.redraw(whence=3)
-
+        self.set_onscreen_message(text, redraw=redraw)
         if delay:
             # schedule a call to onscreen_message_off after
             # `delay` sec
             pass
 
-    def onscreen_message_off(self):
-        return self.onscreen_message(None)
-
-    def show_pan_mark(self, tf):
-        self.t_.set(show_pan_position=tf)
-        self.redraw(whence=3)
-
 
 class ImageViewEvent(ImageViewMock):
 
@@ -261,20 +225,14 @@ class ImageViewEvent(ImageViewMock):
         # gestures
         #...
 
-        # last known window mouse position
-        self.last_win_x = 0
-        self.last_win_y = 0
-        # last known data mouse position
-        self.last_data_x = 0
-        self.last_data_y = 0
         # Does widget accept focus when mouse enters window
         self.enter_focus = self.t_.get('enter_focus', True)
 
         # Define cursors
         for curname, filename in (('pan', 'openHandCursor.png'),
-                               ('pick', 'thinCrossCursor.png')):
-            path = os.path.join(icon_dir, filename)
-            cur = make_cursor(path, 8, 8)
+                                  ('pick', 'thinCrossCursor.png')):
+            path = os.path.join(icondir, filename)
+            cur = self.make_cursor(path, 8, 8)
             self.define_cursor(curname, cur)
 
         # key table mapping characters produced by the key down events
@@ -386,6 +344,7 @@ class ImageViewEvent(ImageViewMock):
         Adjust method signature as appropriate for callback.
         """
         # x, y = coordinates where the button was pressed
+        self.last_win_x, self.last_win_y = x, y
 
         button = 0
         # Prepare a button mask with bits set as follows:
@@ -396,6 +355,8 @@ class ImageViewEvent(ImageViewMock):
         self.logger.debug("button down event at %dx%d, button=%x" % (x, y, button))
 
         data_x, data_y = self.get_data_xy(x, y)
+        self.last_data_x, self.last_data_y = data_x, data_y
+
         return self.make_ui_callback('button-press', button, data_x, data_y)
 
     def button_release_event(self, widget, event):
@@ -404,19 +365,16 @@ class ImageViewEvent(ImageViewMock):
         Adjust method signature as appropriate for callback.
         """
         # x, y = coordinates where the button was released
+        self.last_win_x, self.last_win_y = x, y
 
         button = 0
         # prepare button mask as in button_press_event()
 
         data_x, data_y = self.get_data_xy(x, y)
+        self.last_data_x, self.last_data_y = data_x, data_y
+
         return self.make_ui_callback('button-release', button, data_x, data_y)
 
-    def get_last_win_xy(self):
-        return (self.last_win_x, self.last_win_y)
-
-    def get_last_data_xy(self):
-        return (self.last_data_x, self.last_data_y)
-
     def motion_notify_event(self, widget, event):
         """
         Called when a mouse cursor is moving in the widget.
@@ -525,11 +483,4 @@ class CanvasView(ImageViewZoom):
         self.objects[0] = self.private_canvas
 
 
-def make_cursor(iconpath, x, y):
-    # return a cursor in the widget set's instance type
-    # iconpath usually refers to a PNG file and x/y is the
-    # cursor hot spot
-    cursorw = None
-    return cursorw
-
 #END
