@@ -21,6 +21,7 @@ from ginga import ImageView
 from ginga import Mixins, Bindings, colors
 from . import transform
 from ginga.mplw.CanvasRenderMpl import CanvasRenderer
+from ginga.mplw.MplHelp import Timer
 
 # Override some matplotlib keyboard UI defaults
 rc = matplotlib.rcParams
@@ -65,24 +66,16 @@ class ImageViewMpl(ImageView.ImageViewBase):
         self.img_fg = (1.0, 1.0, 1.0)
         self.img_bg = (0.5, 0.5, 0.5)
 
-        self.message = None
-
         self.in_axes = False
         # Matplotlib expects RGBA data for color images
         self._rgb_order = 'RGBA'
 
         self.renderer = CanvasRenderer(self)
 
-        # cursors
-        self.cursor = {}
-
         # For timing events
         self._msg_timer = None
         self._defer_timer = None
 
-        self.t_.setDefaults(show_pan_position=False,
-                            onscreen_ff='Sans Serif')
-
     def set_figure(self, figure):
         """Call this with the matplotlib Figure() object."""
         self.figure = figure
@@ -115,21 +108,13 @@ class ImageViewMpl(ImageView.ImageViewBase):
         self._defer_timer = None
 
         if hasattr(figure.canvas, 'new_timer'):
-            self._msg_timer = figure.canvas.new_timer()
-            self._msg_timer.single_shot = True
-            self._msg_timer.add_callback(self.onscreen_message_off)
+            self._msg_timer = Timer(0.0,
+                                    lambda timer: self.onscreen_message(None),
+                                    mplcanvas=figure.canvas)
 
-            self._defer_timer = figure.canvas.new_timer()
-            self._defer_timer.single_shot = True
-            self._defer_timer.add_callback(self.delayed_redraw)
-
-        # marker drawn at the center of the image for debugging
-        self.cross1 = lines.Line2D((0.49, 0.51), (0.50, 0.50),
-                                   transform=newax.transAxes,
-                                   color='red', alpha=1.0)
-        self.cross2 = lines.Line2D((0.50, 0.50), (0.49, 0.51),
-                                    transform=newax.transAxes,
-                                    color='red', alpha=1.0)
+            self._defer_timer = Timer(0.0,
+                                      lambda timer: self.delayed_redraw(),
+                                      mplcanvas=figure.canvas)
 
         canvas = figure.canvas
         if hasattr(canvas, 'viewer'):
@@ -260,15 +245,6 @@ class ImageViewMpl(ImageView.ImageViewBase):
         # clear utility axis
         self.ax_util.cla()
 
-        # Draw a cross in the center of the window in debug mode
-        if self.t_['show_pan_position']:
-            self.ax_util.add_line(self.cross1)
-            self.ax_util.add_line(self.cross2)
-
-        # render message if there is one currently
-        if self.message:
-            self.draw_message(self.message)
-
         # force an update of the figure
         self.figure.canvas.draw()
 
@@ -282,14 +258,6 @@ class ImageViewMpl(ImageView.ImageViewBase):
         ##     ax.set_xlim(x0, x1)
         ##     ax.set_ylim(y0, y1)
 
-    def draw_message(self, message):
-        # r, g, b = self.img_fg
-        self.ax_util.text(0.5, 0.33, message,
-                          fontsize=24, horizontalalignment='center',
-                          color='white',
-                          verticalalignment='center',
-                          transform=self.ax_util.transAxes)
-
     def configure_window(self, width, height):
         self.configure(width, height)
 
@@ -321,60 +289,23 @@ class ImageViewMpl(ImageView.ImageViewBase):
     def set_cursor(self, cursor):
         pass
 
-    def define_cursor(self, ctype, cursor):
-        self.cursor[ctype] = cursor
+    def onscreen_message(self, text, delay=None, redraw=True):
+        if self._msg_timer is not None:
+            self._msg_timer.cancel()
 
-    def get_cursor(self, ctype):
-        return self.cursor[ctype]
+        self.set_onscreen_message(text, redraw=redraw)
 
-    def switch_cursor(self, ctype):
-        self.set_cursor(self.cursor[ctype])
-
-    def set_fg(self, r, g, b):
-        self.img_fg = (r, g, b)
-        self.redraw(whence=3)
-
-    def onscreen_message(self, text, delay=None):
-        try:
-            self._msg_timer.stop()
-        except:
-            pass
-
-        self.message = text
-        self.redraw(whence=3)
-
-        if delay:
-            time_ms = int(delay * 1000.0)
-            self._msg_timer.interval = time_ms
-            self._msg_timer.start()
-
-    def onscreen_message_off(self):
-        return self.onscreen_message(None)
+        if self._msg_timer is not None and delay is not None:
+            self._msg_timer.start(delay)
 
     def reschedule_redraw(self, time_sec):
-
         if self._defer_timer is None:
             self.delayed_redraw()
             return
 
-        try:
-            self._defer_timer.stop()
-        except:
-            pass
+        self._defer_timer.cancel()
+        self._defer_timer.start(time_sec)
 
-        time_ms = int(time_sec * 1000)
-        try:
-            self._defer_timer.interval = time_ms
-            self._defer_timer.start()
-
-        except Exception as e:
-            self.logger.warning("Exception starting timer: %s; "
-                             "using unoptomized redraw" % (str(e)))
-            self.delayed_redraw()
-
-    def show_pan_mark(self, tf):
-        self.t_.set(show_pan_position=tf)
-        self.redraw(whence=3)
 
 class ImageViewEvent(ImageViewMpl):
 
@@ -382,12 +313,6 @@ class ImageViewEvent(ImageViewMpl):
         ImageViewMpl.__init__(self, logger=logger, rgbmap=rgbmap,
                               settings=settings)
 
-        # last known window mouse position
-        self.last_win_x = 0
-        self.last_win_y = 0
-        # last known data mouse position
-        self.last_data_x = 0
-        self.last_data_y = 0
         # Does widget accept focus when mouse enters window
         self.enter_focus = self.t_.get('enter_focus', True)
 
@@ -502,30 +427,32 @@ class ImageViewEvent(ImageViewMpl):
 
     def button_press_event(self, event):
         x, y = event.x, event.y
+        self.last_win_x, self.last_win_y = x, y
+
         button = 0
         if event.button in (1, 2, 3):
             button |= 0x1 << (event.button - 1)
         self.logger.debug("button event at %dx%d, button=%x" % (x, y, button))
 
         data_x, data_y = self.get_data_xy(x, y)
+        self.last_data_x, self.last_data_y = data_x, data_y
+
         return self.make_ui_callback('button-press', button, data_x, data_y)
 
     def button_release_event(self, event):
         x, y = event.x, event.y
+        self.last_win_x, self.last_win_y = x, y
+
         button = 0
         if event.button in (1, 2, 3):
             button |= 0x1 << (event.button - 1)
         self.logger.debug("button release at %dx%d button=%x" % (x, y, button))
 
         data_x, data_y = self.get_data_xy(x, y)
+        self.last_data_x, self.last_data_y = data_x, data_y
+
         return self.make_ui_callback('button-release', button, data_x, data_y)
 
-    def get_last_win_xy(self):
-        return (self.last_win_x, self.last_win_y)
-
-    def get_last_data_xy(self):
-        return (self.last_data_x, self.last_data_y)
-
     def motion_notify_event(self, event):
         button = 0
         x, y = event.x, event.y
@@ -543,6 +470,7 @@ class ImageViewEvent(ImageViewMpl):
 
     def scroll_event(self, event):
         x, y = event.x, event.y
+        self.last_win_x, self.last_win_y = x, y
 
         # Matplotlib only gives us the number of steps of the scroll,
         # positive for up and negative for down.  No horizontal scrolling.
