@@ -10,14 +10,12 @@ import numpy
 from io import BytesIO
 
 from ginga.qtw.QtHelp import QtGui, QtCore, QFont, QColor, QImage, \
-     QPixmap, QCursor, QPainter, have_pyqt5, get_scroll_info
+     QPixmap, QCursor, QPainter, have_pyqt5, Timer, get_scroll_info
 from ginga import ImageView, Mixins, Bindings
 import ginga.util.six as six
 from ginga.util.six.moves import map, zip
 from ginga.qtw.CanvasRenderQt import CanvasRenderer
-
-moduleHome = os.path.split(sys.modules[__name__].__file__)[0]
-icon_dir = os.path.abspath(os.path.join(moduleHome, '..', 'icons'))
+from ginga.util.paths import icondir
 
 
 class ImageViewQtError(ImageView.ImageViewError):
@@ -131,22 +129,10 @@ class ImageViewQt(ImageView.ImageViewBase):
 
         self.renderer = CanvasRenderer(self)
 
-        self.t_.setDefaults(show_pan_position=False,
-                            onscreen_ff='Sans Serif')
-
-        self.message = None
-        self.msgtimer = QtCore.QTimer()
-        self.msgtimer.timeout.connect(self.onscreen_message_off)
-        self.msgfont = QFont(self.t_['onscreen_ff'],
-                                   pointSize=24)
-        # cursors
-        self.cursor = {}
+        self.msgtimer = Timer(0.0, lambda timer: self.onscreen_message_off())
 
         # For optomized redrawing
-        self._defer_task = QtCore.QTimer()
-        self._defer_task.setSingleShot(True)
-        self._defer_task.timeout.connect(self.delayed_redraw)
-
+        self._defer_task = Timer(0.0, lambda timer: self.delayed_redraw())
 
     def get_widget(self):
         return self.imgwin
@@ -174,33 +160,6 @@ class ImageViewQt(ImageView.ImageViewBase):
                           qimage,
                           QtCore.QRect(0, 0, width, height))
 
-        # Draw a cross in the center of the window in debug mode
-        if self.t_['show_pan_position']:
-            clr = QColor()
-            clr.setRgbF(1.0, 0.0, 0.0)
-            painter.setPen(clr)
-            ctr_x, ctr_y = self.get_center()
-            painter.drawLine(ctr_x - 10, ctr_y, ctr_x + 10, ctr_y)
-            painter.drawLine(ctr_x, ctr_y - 10, ctr_x, ctr_y + 10)
-
-        # render self.message
-        if self.message:
-            self._draw_message(painter, imgwin_wd, imgwin_ht,
-                               self.message)
-
-    def _draw_message(self, painter, width, height, message):
-        fgclr = self._get_color(*self.img_fg)
-        painter.setPen(fgclr)
-        painter.setBrush(fgclr)
-        painter.setFont(self.msgfont)
-        rect = painter.boundingRect(0, 0, 1000, 1000, 0, message)
-        x1, y1, x2, y2 = rect.getCoords()
-        wd = x2 - x1
-        ht = y2 - y1
-        y = ((height // 3) * 2) - (ht // 2)
-        x = (width // 2) - (wd // 2)
-        painter.drawText(x, y, message)
-
 
     def render_image(self, rgbobj, dst_x, dst_y):
         """Render the image represented by (rgbobj) at dst_x, dst_y
@@ -222,7 +181,11 @@ class ImageViewQt(ImageView.ImageViewBase):
         self.logger.debug("window size reconfigured to %dx%d" % (
             width, height))
         if hasattr(self, 'scene'):
-            self.scene.setSceneRect(0, 0, width, height)
+            # By default, a QGraphicsView comes with a 1-pixel margin
+            # You will get scrollbars unless you account for this
+            # See http://stackoverflow.com/questions/3513788/qt-qgraphicsview-without-scrollbar
+            width, height = width - 2, height - 2
+            self.scene.setSceneRect(1, 1, width-2, height-2)
         # If we need to build a new pixmap do it here.  We allocate one
         # twice as big as necessary to prevent having to reinstantiate it
         # all the time.  On Qt this causes unpleasant flashing in the display.
@@ -250,10 +213,6 @@ class ImageViewQt(ImageView.ImageViewBase):
         qbuf.close()
         return ibuf
 
-    def get_rgb_image_as_bytes(self, format='png', quality=90):
-        ibuf = self.get_rgb_image_as_buffer(format=format, quality=quality)
-        return bytes(ibuf.getvalue())
-
     def get_rgb_image_as_widget(self, output=None, format='png',
                                 quality=90):
         imgwin_wd, imgwin_ht = self.get_window_size()
@@ -265,7 +224,7 @@ class ImageViewQt(ImageView.ImageViewBase):
         qimg = self.get_rgb_image_as_widget()
         res = qimg.save(filepath, format=format, quality=quality)
 
-    def get_image_as_widget(self):
+    def get_plain_image_as_widget(self):
         """Used for generating thumbnails.  Does not include overlaid
         graphics.
         """
@@ -273,21 +232,16 @@ class ImageViewQt(ImageView.ImageViewBase):
         image = self._get_qimage(arr)
         return image
 
-    def save_image_as_file(self, filepath, format='png', quality=90):
+    def save_plain_image_as_file(self, filepath, format='png', quality=90):
         """Used for generating thumbnails.  Does not include overlaid
         graphics.
         """
-        qimg = self.get_image_as_widget()
+        qimg = self.get_plain_image_as_widget()
         res = qimg.save(filepath, format=format, quality=quality)
 
     def reschedule_redraw(self, time_sec):
-        try:
-            self._defer_task.stop()
-        except:
-            pass
-
-        time_ms = int(time_sec * 1000)
-        self._defer_task.start(time_ms)
+        self._defer_task.cancel()
+        self._defer_task.start(time_sec)
 
     def update_image(self):
         if (not self.pixmap) or (not self.imgwin):
@@ -306,11 +260,11 @@ class ImageViewQt(ImageView.ImageViewBase):
         if self.imgwin is not None:
             self.imgwin.setCursor(cursor)
 
-    def define_cursor(self, ctype, cursor):
-        self.cursor[ctype] = cursor
-
-    def get_cursor(self, ctype):
-        return self.cursor[ctype]
+    def make_cursor(self, iconpath, x, y):
+        image = QImage()
+        image.load(iconpath)
+        pm = QPixmap(image)
+        return QCursor(pm, x, y)
 
     def center_cursor(self):
         if self.imgwin is None:
@@ -337,9 +291,6 @@ class ImageViewQt(ImageView.ImageViewBase):
     def get_rgb_order(self):
         return self._rgb_order
 
-    def switch_cursor(self, ctype):
-        self.set_cursor(self.cursor[ctype])
-
     def _get_qimage(self, bgra):
         h, w, channels = bgra.shape
 
@@ -354,23 +305,11 @@ class ImageViewQt(ImageView.ImageViewBase):
         clr = QColor(int(r*n), int(g*n), int(b*n))
         return clr
 
-    def onscreen_message(self, text, delay=None):
-        try:
-            self.msgtimer.stop()
-        except:
-            pass
-        self.message = text
-        self.redraw(whence=3)
-        if delay:
-            ms = int(delay * 1000.0)
-            self.msgtimer.start(ms)
-
-    def onscreen_message_off(self):
-        return self.onscreen_message(None)
-
-    def show_pan_mark(self, tf):
-        self.t_.set(show_pan_position=tf)
-        self.redraw(whence=3)
+    def onscreen_message(self, text, delay=None, redraw=True):
+        self.msgtimer.cancel()
+        self.set_onscreen_message(text, redraw=redraw)
+        if delay is not None:
+            self.msgtimer.start(delay)
 
 
 class RenderMixin(object):
@@ -477,20 +416,14 @@ class ImageViewEvent(ImageViewQt):
         #imgwin.grabGesture(QtCore.Qt.TapGesture)
         #imgwin.grabGesture(QtCore.Qt.TapAndHoldGesture)
 
-        # last known window mouse position
-        self.last_win_x = 0
-        self.last_win_y = 0
-        # last known data mouse position
-        self.last_data_x = 0
-        self.last_data_y = 0
         # Does widget accept focus when mouse enters window
         self.enter_focus = self.t_.get('enter_focus', True)
 
         # Define cursors
         for curname, filename in (('pan', 'openHandCursor.png'),
                                ('pick', 'thinCrossCursor.png')):
-            path = os.path.join(icon_dir, filename)
-            cur = make_cursor(path, 8, 8)
+            path = os.path.join(icondir, filename)
+            cur = self.make_cursor(path, 8, 8)
             self.define_cursor(curname, cur)
 
         # @$%&^(_)*&^ qt!!
@@ -607,6 +540,7 @@ class ImageViewEvent(ImageViewQt):
     def button_press_event(self, widget, event):
         buttons = event.buttons()
         x, y = event.x(), event.y()
+        self.last_win_x, self.last_win_y = x, y
 
         button = 0
         if buttons & QtCore.Qt.LeftButton:
@@ -618,12 +552,15 @@ class ImageViewEvent(ImageViewQt):
         self.logger.debug("button down event at %dx%d, button=%x" % (x, y, button))
 
         data_x, data_y = self.get_data_xy(x, y)
+        self.last_data_x, self.last_data_y = data_x, data_y
+
         return self.make_ui_callback('button-press', button, data_x, data_y)
 
     def button_release_event(self, widget, event):
         # note: for mouseRelease this needs to be button(), not buttons()!
         buttons = event.button()
         x, y = event.x(), event.y()
+        self.last_win_x, self.last_win_y = x, y
 
         button = 0
         if buttons & QtCore.Qt.LeftButton:
@@ -634,15 +571,21 @@ class ImageViewEvent(ImageViewQt):
             button |= 0x4
 
         data_x, data_y = self.get_data_xy(x, y)
+        self.last_data_x, self.last_data_y = data_x, data_y
+
         return self.make_ui_callback('button-release', button, data_x, data_y)
 
-    def get_last_win_xy(self):
-        return (self.last_win_x, self.last_win_y)
-
-    def get_last_data_xy(self):
-        return (self.last_data_x, self.last_data_y)
-
     def motion_notify_event(self, widget, event):
+
+        if self.is_redraw_pending():
+            # NOTE: this hack works around a bug in Qt5 (ver < 5.6.2)
+            # where mouse motion events are not compressed properly,
+            # causing many events to build up in the queue and slowing
+            # down the viewer due to each one forcing a redraw.
+            # This test tells us there is a deferred redraw waiting;
+            # discard any motion events until the redraw happens.
+            return True
+
         buttons = event.buttons()
         x, y = event.x(), event.y()
         self.last_win_x, self.last_win_y = x, y
@@ -664,15 +607,15 @@ class ImageViewEvent(ImageViewQt):
         x, y = event.x(), event.y()
         self.last_win_x, self.last_win_y = x, y
 
-        numDegrees, direction = get_scroll_info(event)
-        self.logger.debug("scroll deg=%f direction=%f" % (
-            numDegrees, direction))
+        num_degrees, direction = get_scroll_info(event)
+        self.logger.debug("scroll deg={} direction={}".format(
+            num_degrees, direction))
 
         data_x, data_y = self.get_data_xy(x, y)
         self.last_data_x, self.last_data_y = data_x, data_y
 
-        return self.make_ui_callback('scroll', direction, numDegrees,
-                                  data_x, data_y)
+        return self.make_ui_callback('scroll', direction, num_degrees,
+                                     data_x, data_y)
 
     def gesture_event(self, widget, event):
         gesture = event.gestures()[0]
@@ -854,11 +797,135 @@ class CanvasView(ImageViewZoom):
         self.objects[0] = self.private_canvas
 
 
-def make_cursor(iconpath, x, y):
-    image = QImage()
-    image.load(iconpath)
-    pm = QPixmap(image)
-    return QCursor(pm, x, y)
+class ScrolledView(QtGui.QAbstractScrollArea):
+    """A class that can take a viewer as a parameter and add scroll bars
+    that respond to the pan/zoom levels.
+    """
+
+    def __init__(self, viewer, parent=None):
+        self.viewer = viewer
+        super(ScrolledView, self).__init__(parent=parent)
+
+        # the window jiggles annoyingly as the scrollbar is alternately
+        # shown and hidden if we use the default "as needed" policy, so
+        # default to always showing them (user can change this after
+        # calling the constructor, if desired)
+        self.scroll_bars(horizontal='on', vertical='on')
+
+        self._adjusting = False
+        self._scrolling = False
+        self.pad = 20
+        self.upper_h = 100
+        self.upper_v = 100
+
+        # reparent the viewer widget
+        vp = self.viewport()
+        self.v_w = viewer.get_widget()
+        self.v_w.setParent(vp)
+
+        hsb = self.horizontalScrollBar()
+        hsb.setTracking(True)
+        hsb.setRange(0, 100)
+        hsb.setSingleStep(1)
+        vsb = self.verticalScrollBar()
+        vsb.setRange(0, 100)
+        vsb.setSingleStep(1)
+        vsb.setTracking(True)
+
+        self.viewer.add_callback('redraw', self._calc_scrollbars)
+        self.viewer.add_callback('limits-set',
+                                 lambda v, l: self._calc_scrollbars(v))
+
+    def get_widget(self):
+        return self
+
+    def resizeEvent(self, event):
+        """Override from QAbstractScrollArea.
+        Resize the viewer widget when the viewport is resized."""
+        vp = self.viewport()
+        rect = vp.geometry()
+        x1, y1, x2, y2 = rect.getCoords()
+        width = x2 - x1
+        height = y2 - y1
+
+        self.v_w.resize(width, height)
+
+    def _calc_scrollbars(self, viewer):
+        """Calculate and set the scrollbar handles from the pan and
+        zoom positions.
+        """
+        if self._scrolling:
+            return
+
+        # flag that suppresses a cyclical callback
+        self._adjusting = True
+        try:
+            bd = self.viewer.get_bindings()
+            res = bd.calc_pan_pct(self.viewer, pad=self.pad)
+            if res is None:
+                return
+
+            hsb = self.horizontalScrollBar()
+            vsb = self.verticalScrollBar()
+
+            page_h, page_v = (int(round(res.thm_pct_x * 100)),
+                              int(round(res.thm_pct_y * 100)))
+            hsb.setPageStep(page_h)
+            vsb.setPageStep(page_v)
+
+            upper_h, upper_v = max(1, 100 - page_h), max(1, 100 - page_v)
+            hsb.setRange(0, upper_h)
+            vsb.setRange(0, upper_v)
+            self.upper_h, self.upper_v = upper_h, upper_v
+
+            val_h, val_v = (int(round(res.pan_pct_x * self.upper_h)),
+                            int(round((1.0 - res.pan_pct_y) * self.upper_v)))
+            hsb.setValue(val_h)
+            vsb.setValue(val_v)
+
+        finally:
+            self._adjusting = False
+
+    def scrollContentsBy(self, dx, dy):
+        """Override from QAbstractScrollArea.
+        Called when the scroll bars are adjusted by the user.
+        """
+        if self._adjusting:
+            return
+
+        self._scrolling = True
+        try:
+            hsb = self.horizontalScrollBar()
+            vsb = self.verticalScrollBar()
+
+            pct_x = hsb.value() / float(self.upper_h)
+            # invert Y pct because of orientation of scrollbar
+            pct_y = 1.0 - (vsb.value() / float(self.upper_v))
+
+            bd = self.viewer.get_bindings()
+            bd.pan_by_pct(self.viewer, pct_x, pct_y, pad=self.pad)
+
+        finally:
+            self._scrolling = False
+
+    def scroll_bars(self, horizontal='on', vertical='on'):
+        if horizontal == 'on':
+            self.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
+        elif horizontal == 'off':
+            self.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
+        elif horizontal == 'auto':
+            self.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
+        else:
+            raise ValueError("Bad scroll bar option: '%s'; should be one of ('on', 'off' or 'auto')" % (horizontal))
+
+        if vertical == 'on':
+            self.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOn)
+        elif vertical == 'off':
+            self.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAlwaysOff)
+        elif vertical == 'auto':
+            self.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
+        else:
+            raise ValueError("Bad scroll bar option: '%s'; should be one of ('on', 'off' or 'auto')" % (vertical))
 
 
 #END
