@@ -1,9 +1,6 @@
 #
 # ImageViewTk.py -- a backend for Ginga using a Tk canvas widget
 #
-# Eric Jeschke (eric@naoj.org)
-#
-# Copyright (c)  Eric R. Jeschke.  All rights reserved.
 # This is open-source software licensed under a BSD license.
 # Please see the file LICENSE.txt for details.
 
@@ -15,6 +12,7 @@ import PIL.ImageTk as PILimageTk
 from ginga import Mixins, Bindings, colors
 from ginga.canvas.mixins import DrawingMixin, CanvasMixin, CompoundMixin
 from ginga.util.toolbox import ModeIndicator
+from ginga.tkw import TkHelp
 
 try:
     # See if we have aggdraw module--best choice
@@ -52,10 +50,8 @@ class ImageViewTk(ImageView):
         self.tkcanvas = None
         self.tkphoto = None
 
-        self.msgtask = None
-
-        # see reschedule_redraw() method
         self._defer_task = None
+        self.msgtask = None
 
 
     def set_widget(self, canvas):
@@ -68,6 +64,14 @@ class ImageViewTk(ImageView):
         width = canvas.winfo_width()
         height = canvas.winfo_height()
 
+        # see reschedule_redraw() method
+        self._defer_task = TkHelp.Timer(0.0,
+                                        lambda timer: self.delayed_redraw(),
+                                        tkcanvas=canvas)
+        self.msgtask = TkHelp.Timer(0.0,
+                                    lambda timer: self.onscreen_message(None),
+                                    tkcanvas=canvas)
+
         self.configure_window(width, height)
 
     def get_widget(self):
@@ -101,14 +105,8 @@ class ImageViewTk(ImageView):
         cr.config(scrollregion=cr.bbox('all'))
 
     def reschedule_redraw(self, time_sec):
-        if self.tkcanvas is not None:
-            try:
-                self.tkcanvas.after_cancel(self._defer_task)
-            except:
-                pass
-            time_ms = int(time_sec * 1000)
-            self._defer_task = self.tkcanvas.after(time_ms,
-                                                   self.delayed_redraw)
+        self._defer_task.cancel()
+        self._defer_task.start(time_sec)
 
     def configure_window(self, width, height):
         self.configure_surface(width, height)
@@ -121,20 +119,13 @@ class ImageViewTk(ImageView):
             return
         self.tkcanvas.config(cursor=cursor)
 
-    def onscreen_message(self, text, delay=None):
+    def onscreen_message(self, text, delay=None, redraw=True):
         if self.tkcanvas is None:
             return
-        if self.msgtask:
-            try:
-                self.tkcanvas.after_cancel(self.msgtask)
-            except:
-                pass
-        self.message = text
-        self.redraw(whence=3)
-        if delay:
-            ms = int(delay * 1000.0)
-            self.msgtask = self.tkcanvas.after(ms,
-                                              lambda: self.onscreen_message(None))
+        self.msgtask.cancel()
+        self.set_onscreen_message(text, redraw=redraw)
+        if delay is not None:
+            self.msgtask.start(delay)
 
 
 class ImageViewEvent(ImageViewTk):
@@ -143,12 +134,6 @@ class ImageViewEvent(ImageViewTk):
         ImageViewTk.__init__(self, logger=logger, rgbmap=rgbmap,
                              settings=settings)
 
-        # last known window mouse position
-        self.last_win_x = 0
-        self.last_win_y = 0
-        # last known data mouse position
-        self.last_data_x = 0
-        self.last_data_y = 0
         # Does widget accept focus when mouse enters window
         self.enter_focus = self.t_.get('enter_focus', True)
 
@@ -293,6 +278,8 @@ class ImageViewEvent(ImageViewTk):
 
     def button_press_event(self, event):
         x = event.x; y = event.y
+        self.last_win_x, self.last_win_y = x, y
+
         button = 0
         if event.num != 0:
             # some kind of wierd convention for scrolling, shoehorned into
@@ -318,11 +305,15 @@ class ImageViewEvent(ImageViewTk):
         self.logger.debug("button event at %dx%d, button=%x" % (x, y, button))
 
         data_x, data_y = self.get_data_xy(x, y)
+        self.last_data_x, self.last_data_y = data_x, data_y
+
         return self.make_ui_callback('button-press', button, data_x, data_y)
 
     def button_release_event(self, event):
         # event.button, event.x, event.y
         x = event.x; y = event.y
+        self.last_win_x, self.last_win_y = x, y
+
         button = 0
         if event.num != 0:
             if event.num in (4, 5):
@@ -333,14 +324,10 @@ class ImageViewEvent(ImageViewTk):
         self.logger.debug("button release at %dx%d button=%x" % (x, y, button))
 
         data_x, data_y = self.get_data_xy(x, y)
+        self.last_data_x, self.last_data_y = data_x, data_y
+
         return self.make_ui_callback('button-release', button, data_x, data_y)
 
-    def get_last_win_xy(self):
-        return (self.last_win_x, self.last_win_y)
-
-    def get_last_data_xy(self):
-        return (self.last_data_x, self.last_data_y)
-
     def motion_notify_event(self, event):
         #button = 0
         button = self._button
