@@ -1,15 +1,19 @@
 #
-# path.py -- path information
+# paths.py -- path information
 # 
-# Copyright (c) Eric R. Jeschke.  All rights reserved.
 # This is open-source software licensed under a BSD license.
 # Please see the file LICENSE.txt for details.
 #
 import os
+import ginga.icons
 
 home = None
 ginga_home = None
 
+# path to our icons
+icondir = os.path.split(ginga.icons.__file__)[0]
+
+
 if 'GINGA_HOME' in os.environ:
     # User override
     ginga_home = os.environ['GINGA_HOME']
