@@ -1,9 +1,6 @@
 #
 # PgHelp.py -- web application threading help routines.
 #
-# Eric Jeschke (eric@naoj.org)
-#
-# Copyright (c) Eric R. Jeschke.  All rights reserved.
 # This is open-source software licensed under a BSD license.
 # Please see the file LICENSE.txt for details.
 #
@@ -172,6 +169,49 @@ class WindowHandler(tornado.web.RequestHandler):
 
         self.write(output)
 
+
+class Timer(object):
+    """Abstraction of a GUI-toolkit implemented timer."""
+
+    def __init__(self, ival_sec, expire_cb, data=None, mplcanvas=None):
+        """Create a timer set to expire after `ival_sec` and which will
+        call the callable `expire_cb` when it expires.
+        """
+        self.ival_sec = ival_sec
+        self.cb = expire_cb
+        self.data = data
+
+        self._timer = mplcanvas.new_timer()
+        self._timer.single_shot = True
+        self._timer.add_callback(self._redirect_cb)
+
+    def start(self, ival_sec=None):
+        """Start the timer.  If `ival_sec` is not None, it should
+        specify the time to expiration in seconds.
+        """
+        if ival_sec is None:
+            ival_sec = self.ival_sec
+
+        self.cancel()
+
+        # Matplotlib timer set in milliseconds
+        time_ms = int(ival_sec * 1000.0)
+        self._timer.interval = time_ms
+        self._timer.start()
+
+    def _redirect_cb(self):
+        self.cb(self)
+
+    def cancel(self):
+        """Cancel this timer.  If the timer is not running, there
+        is no error.
+        """
+        try:
+            self._timer.stop()
+        except:
+            pass
+
+
 def get_image_src_from_buffer(img_buf, imgtype='png'):
     img_string = binascii.b2a_base64(img_buf)
     if isinstance(img_string, bytes):
