@@ -11,10 +11,6 @@ from functools import reduce
 
 from ginga.misc import Callback, Bunch, LineHistory
 from ginga.web.pgw import PgHelp
-import ginga.icons
-
-# path to our icons
-icondir = os.path.split(ginga.icons.__file__)[0]
 
 class WidgetError(Exception):
     """For errors thrown in this module."""
@@ -95,6 +91,10 @@ class WidgetBase(Callback.Callbacks):
     def get_size(self):
         return self.width, self.height
 
+    def get_pos(self):
+        # TODO
+        return 0, 0
+
     def delete(self):
         # for now...
         pass
@@ -236,6 +236,7 @@ class TextArea(WidgetBase):
 
         if not autoscroll:
             return
+        app.do_operation('scroll_bottom', id=self.id)
 
     def get_text(self):
         return self.text
@@ -692,8 +693,8 @@ class StatusBar(Label):
     def __init__(self):
         super(StatusBar, self).__init__()
 
-    def set_message(self, msg_str):
-        # TODO: remove message in about 10 seconds
+    def set_message(self, msg_str, duration=10.0):
+        # TODO: remove message in about `duration` seconds
         self.set_text(msg_str)
 
 
@@ -1212,14 +1213,17 @@ class Toolbar(ContainerBase):
         self.orientation = orientation
         self.widget = Box(orientation=orientation)
 
-    def add_action(self, text, toggle=False, iconpath=None):
+    def add_action(self, text, toggle=False, iconpath=None, iconsize=None):
         child = ToolbarAction()
         self.text = text
         if iconpath:
-            native_image = PgHelp.get_icon(iconpath, size=(24, 24),
+            wd, ht = 24, 24
+            if iconsize is not None:
+                wd, ht = iconsize
+            native_image = PgHelp.get_icon(iconpath, size=(wd, ht),
                                            format='png')
             widget = Image(native_image=native_image)
-            widget.resize(24, 24)
+            widget.resize(wd, ht)
         else:
             widget = Button(text)
         child.checkable = toggle
