@@ -20,6 +20,7 @@ Tested with Chromium 41.0.2272.76, Firefox 37.0.2, Safari 7.1.6
 """
 from __future__ import print_function
 import sys, os
+import math
 import logging
 import threading
 
@@ -30,6 +31,9 @@ from ginga import AstroImage, colors
 from ginga.canvas.CanvasObject import get_canvas_types
 from ginga.misc import log, Task
 from ginga.util import catalog, iohelper
+from ginga.Bindings import ImageViewBindings
+from ginga.misc.Settings import SettingGroup
+from ginga.util.paths import ginga_home
 
 from ginga.web.pgw import templates, js, PgHelp, Widgets, Viewers
 
@@ -55,6 +59,11 @@ class EnhancedCanvasView(Viewers.CanvasView):
         """
         Capture the window of a viewer.
         """
+        # force any delayed redraws
+        # TODO: this really needs to be addressed in get_rgb_image_as_bytes()
+        # of the various superclasses, as it affects other backends as well
+        self.redraw_now()
+
         from IPython.display import Image
         return Image(data=bytes(self.get_rgb_image_as_bytes(format='png')),
                      format='png', embed=True)
@@ -96,6 +105,7 @@ class EnhancedCanvasView(Viewers.CanvasView):
         # enable drawing on the canvas
         canvas.enable_draw(True)
         canvas.enable_edit(True)
+        canvas.set_drawtype(None)
         canvas.ui_setActive(True)
         canvas.set_surface(self)
         canvas.register_for_cursor_drawing(self)
@@ -104,25 +114,15 @@ class EnhancedCanvasView(Viewers.CanvasView):
 
         return canvas
 
-    def resize(self, width, height):
-        # this shouldn't be needed
-        self.configure_window(width, height)
-
-        self.pgcanvas.resize(width, height)
-
-        # hack to force a browser reload
-        app = self.pgcanvas.get_app()
-        app.do_operation('reload_page', id=self.pgcanvas.id)
-
 class ImageViewer(object):
 
-    def __init__(self, logger, window, viewer_class=None):
+    def __init__(self, logger, window, viewer_class=None, width=512, height=512):
         if viewer_class is None:
             viewer_class = EnhancedCanvasView
         self.logger = logger
         self.url = window.url
-        self.drawcolors = colors.get_colors()
         self.dc = get_canvas_types()
+        self.pixel_base = 1.0
 
         self.top = window
         self.top.add_callback('close', self.closed)
@@ -131,60 +131,49 @@ class ImageViewer(object):
         vbox.set_border_width(2)
         vbox.set_spacing(1)
 
-        fi = viewer_class(logger)
+        # load binding preferences if available
+        cfgfile = os.path.join(ginga_home, "ipg_bindings.cfg")
+        bindprefs = SettingGroup(name='bindings', logger=logger,
+                                 preffile=cfgfile)
+        bindprefs.load(onError='silent')
+
+        bd = ImageViewBindings(logger, settings=bindprefs)
+
+        fi = viewer_class(logger, bindings=bd)
         fi.url = self.url
         fi.enable_autocuts('on')
         fi.set_autocut_params('zscale')
-        fi.enable_autozoom('on')
         fi.set_zoom_algorithm('rate')
-        fi.set_zoomrate(1.4)
-        fi.show_pan_mark(True)
-        fi.set_callback('drag-drop', self.drop_file)
+        fi.set_zoomrate(1.1)
+        fi.enable_autozoom('on')
+        #fi.set_callback('drag-drop', self.drop_file)
         fi.set_callback('none-move', self.motion)
         fi.set_bg(0.2, 0.2, 0.2)
         fi.ui_setActive(True)
         self.fitsimage = fi
+        fi.ipg_parent = self
 
         bd = fi.get_bindings()
         bd.enable_all(True)
 
-        # so trackpad scrolling can be adjusted
-        settings = bd.get_settings()
-        settings.set(scroll_zoom_direct_scale=True)
-
-        # so trackpad scrolling can be adjusted
-        settings = bd.get_settings()
-        settings.set(scroll_zoom_direct_scale=True,
-                     scroll_zoom_acceleration=0.07)
-
         # canvas that we will draw on
         canvas = self.dc.DrawingCanvas()
-        #canvas.enable_draw(True)
-        #canvas.enable_edit(True)
-        #canvas.set_drawtype('rectangle', color='lightblue')
-        canvas.setSurface(fi)
+        canvas.set_surface(fi)
         self.canvas = canvas
         # add canvas to view
         private_canvas = fi.get_canvas()
         private_canvas.add(canvas)
         canvas.ui_setActive(True)
-        #canvas.register_for_cursor_drawing(fi)
         fi.set_canvas(canvas)
-        ## self.drawtypes = canvas.get_drawtypes()
-        ## self.drawtypes.sort()
 
-        # add a color bar
-        private_canvas.add(self.dc.ColorBar(side='bottom', offset=10))
+        fi.set_desired_size(width, height)
+        # force allocation of a surface--may be resized later
+        fi.configure_surface(width, height)
 
         # add little mode indicator that shows modal states in
         # the corner
-        private_canvas.add(self.dc.ModeIndicator(corner='ur', fontsize=14))
-        # little hack necessary to get correct operation of the mode indicator
-        # in all circumstances
-        bm = fi.get_bindmap()
-        bm.add_callback('mode-set', lambda *args: fi.redraw(whence=3))
+        fi.show_mode_indicator(True)
 
-        fi.set_desired_size(512, 512)
         w = Viewers.GingaViewerWidget(viewer=fi)
         vbox.add_widget(w, stretch=1)
 
@@ -196,8 +185,10 @@ class ImageViewer(object):
         slider = Widgets.Slider(orientation='horizontal', dtype=float)
         slider.add_callback('value-changed',
                             lambda w, val: self.adjust_scrolling_accel_cb(val))
-        slider.set_limits(0.0, 12.0, 0.005)
-        slider.set_value(8.0)
+        slider.set_limits(1.0, 9.5, 0.1)
+        val = 4.0
+        slider.set_value(val)
+        self.adjust_scrolling_accel_cb(val)
         hbox.add_widget(slider, stretch=1)
 
         hbox.add_widget(Widgets.Label(''), stretch=1)
@@ -234,7 +225,8 @@ class ImageViewer(object):
         except Exception:
             value = None
 
-        fits_x, fits_y = data_x + 1, data_y + 1
+        pb = self.pixel_base
+        fits_x, fits_y = data_x + pb, data_y + pb
 
         # Calculate WCS RA
         try:
@@ -255,13 +247,19 @@ class ImageViewer(object):
             ra_txt, dec_txt, fits_x, fits_y, value)
         self.readout.set_text(text)
 
+    def set_readout_text(self, text):
+        self.readout.set_text(text)
+
     def adjust_scrolling_accel_cb(self, val):
         def f(x):
-            return (1.0 / 2.0**(10.0-x))
+            return (math.log10(x) / (10 - x) * x/12) + 1.0001
         val2 = f(val)
         self.logger.debug("slider value is %f, setting will be %f" % (val, val2))
-        settings = self.fitsimage.get_bindings().get_settings()
-        settings.set(scroll_zoom_acceleration=val2)
+        # save scale
+        scale_x, scale_y = self.fitsimage.get_scale_xy()
+        self.fitsimage.set_zoomrate(val2)
+        # restore scale
+        self.fitsimage.scale_to(scale_x, scale_y)
         return True
 
     def closed(self, w):
@@ -311,26 +309,34 @@ class ViewerFactory(object):
     def get_threadpool(self):
         return self.thread_pool
 
-    def get_viewer(self, v_id):
+    def get_viewer(self, v_id, viewer_class=None, width=512, height=512,
+                   force_new=False):
         """
         Get an existing viewer by viewer id.  If the viewer does not yet
         exist, make a new one.
         """
-        try:
-            return self.viewers[v_id]
-        except KeyError:
-            pass
+        if not force_new:
+            try:
+                return self.viewers[v_id]
+            except KeyError:
+                pass
 
         #  create top level window
         window = self.app.make_window("Viewer %s" % v_id, wid=v_id)
 
         # our own viewer object, customized with methods (see above)
-        viewer = ImageViewer(self.logger, window)
+        viewer = ImageViewer(self.logger, window,
+                             viewer_class=viewer_class, width=width, height=height)
         #viewer.url = window.url
 
         self.viewers[v_id] = viewer
         return viewer
 
+    def delete_viewer(self, v_id):
+        del self.viewers[v_id]
+
+    def delete_all_viewers(self):
+        self.viewers = {}
 
 class WebServer(object):
 
@@ -348,6 +354,7 @@ class WebServer(object):
         self.ev_quit = ev_quit
 
         self.server = None
+        self.http_server = None
 
     def start(self, use_thread=True, no_ioloop=False):
         self.thread_pool.startall()
@@ -368,7 +375,7 @@ class WebServer(object):
             ],
                factory=self.factory, logger=self.logger)
 
-        self.server.listen(self.port, self.host)
+        self.http_server = self.server.listen(self.port, self.host)
 
         if no_ioloop:
             self.t_ioloop = None
@@ -382,15 +389,23 @@ class WebServer(object):
 
     def stop(self):
         # how to stop tornado server?
-        if not self.t_ioloop is None:
+        if self.t_ioloop is not None:
             self.t_ioloop.stop()
 
         self.thread_pool.stopall()
         self.ev_quit.set()
+        # stop and dereference the tornado server
+        if self.http_server is not None:
+            self.http_server.stop()
+            self.http_server = None
+        self.server = None
 
-    def get_viewer(self, v_id):
+    def get_viewer(self, v_id, viewer_class=None, width=512, height=512,
+                   force_new=False):
         from IPython.display import display, HTML
-        v = self.factory.get_viewer(v_id)
+        v = self.factory.get_viewer(v_id, viewer_class=viewer_class,
+                                    width=width, height=height,
+                                    force_new=force_new)
         url = v.top.url
         viewer = v.fitsimage
         viewer.url = url
